# Copyright (C) 2023 Maxwell G <gotmax@e.email>
#
# SPDX-License-Identifier: GPL-2.0-or-later

from __future__ import annotations

import abc
import argparse
import logging
import re
from collections.abc import Callable, Collection, Iterable, Iterator, Mapping
from contextlib import suppress
from functools import partial
from typing import TYPE_CHECKING, Any, ClassVar, NoReturn, cast

from fedrq._utils import get_source_name

if TYPE_CHECKING:
    from typing_extensions import Self

    from fedrq.backends.base import PackageCompat, RepoqueryBase
else:
    ellipsis = type(...)

LOG = logging.getLogger(__name__)


class FormatterError(Exception):
    pass


_ATTRS: tuple[str, ...] = (
    "name",
    "arch",
    "a",
    "epoch",
    "e",
    "version",
    "v",
    "release",
    "r",
    "from_repo",
    "evr",
    "debug_name",
    "source_name",
    "source_debug_name",
    "installtime",
    "buildtime",
    "size",
    "downloadsize",
    "installsize",
    "provides",
    "requires",
    "recommends",
    "suggests",
    "supplements",
    "enhances",
    "obsoletes",
    "conflicts",
    "sourcerpm",
    "description",
    "summary",
    "license",
    "url",
    "reason",
    "files",
    "reponame",
    "repoid",
    "vendor",
    "packager",
    "location",
)
_MULTILINE_ATTRS = frozenset(
    {
        "description",
        "provides",
        "requires",
        "recommends",
        "suggests",
        "supplements",
        "enhances",
        "obsoletes",
        "conflicts",
        "files",
    }
)
_ATTRS_SINGLE: tuple[str, ...] = tuple(
    set(_ATTRS) - _MULTILINE_ATTRS  # pyright: ignore[reportOperatorIssue]
)
_DEFAULT_MULTILINE_SEPERATOR = "\n---\n"


def _stringify(
    value: Any,
    *,
    multiline_allowed: bool = True,
    multiline_seperator: str = _DEFAULT_MULTILINE_SEPERATOR,
) -> str:
    if value is None or value == "":
        return "(none)"
    if isinstance(value, str) and "\n" in value:
        if not multiline_allowed:
            raise FormatterError("Multiline values are not allowed")
        return value + multiline_seperator
    return str(value)


class Formatter(metaclass=abc.ABCMeta):
    ATTRS = _ATTRS
    MULTILINE = False

    """
    Convert PackageCompat objects into a string representation
    for use with the fedrq CLI.
    """

    def __init__(
        self,
        name: str,
        seperator: str,
        args: str,
        container: Formatters | None = None,
        repoquery: RepoqueryBase | None = None,
    ) -> None:
        self.name = name
        self.seperator = seperator
        self.args = args
        self.container: Formatters = container or Formatters({})
        self.rq = repoquery
        self.validate()

    @abc.abstractmethod
    def format_line(self, package: PackageCompat) -> str:
        """
        Format a single Package object
        """
        pass

    def format(self, packages: Iterable[PackageCompat]) -> Iterable[str]:
        """
        Convert an Iterable of PackageCompat objects
        (or a PackageQueryCompat object) to an Iterable of str.
        """
        yield from map(self.format_line, sorted(packages))

    def err(self, msg: str) -> NoReturn:
        raise FormatterError(f"{self.name!r} FormatterError: {msg}")

    def validate(self) -> None:
        if self.seperator:
            self.err("no arguments are accepted")

    @classmethod
    def frompat(cls, fmt: str, name: str) -> type[Formatter]:
        """
        Convert Python format string (e.g. `{0.name}.{0.arch}`)
        in a Formatter class.
        """

        def format_line(self, package):  # noqa: ARG001
            return fmt.format(package)

        dct = dict(
            __doc__=f"{name} -- {fmt}",
            __module__=__name__,
            format_line=format_line,
        )
        typ = type(f"{name.upper()}Formatter", (cls,), dct)
        return typ

    @classmethod
    def fromfunc(
        cls, func: Callable[[PackageCompat], str], name: str
    ) -> type[Formatter]:
        """
        Convert a function (or other callable) into a Formatter class.
        The function is used as the format_line() method.
        """
        dct = dict(
            format_line=func,
            __module__=__name__,
            __doc__=func.__doc__ if func.__doc__ else name,
        )
        typ = type(f"{name.upper()}Formatter", (cls,), dct)
        return typ

    def __str__(self) -> str:
        return f"{self.name}{self.seperator}{self.args}"


def format_line_notimplemented(
    self: Formatter, package: PackageCompat  # noqa: ARG001
) -> NoReturn:
    raise NotImplementedError


class Formatters(Mapping[str, type[Formatter]]):
    __slots__ = ("__data", "__fallback")
    __data: dict[str, type[Formatter]]
    __fallback: type[Formatter] | None

    """
    Immutable mapping like class of Formatter classes.
    Converts strings and simple functions to Formatter objects.
    Allows merging and adding other Formatters.
    """

    def __new__(
        cls,
        formatters: Mapping[str, Callable | type[Formatter] | str],
        fallback: type[Formatter] | None = None,
    ) -> Self:
        self = super().__new__(cls)
        self.__data = self._formattersv(dict(formatters))
        self.__fallback = fallback
        return self

    @property
    def fallback(self) -> type[Formatter] | None:
        return self.__fallback

    def get_formatter(
        self,
        key: str,
        fallback: type[Formatter] | None | ellipsis = ...,
        *,
        repoquery: RepoqueryBase | None = None,
    ) -> Formatter:
        if fallback is ...:
            fallback = self.fallback
        fallback = cast("type[Formatter] | None", fallback)
        name, seperator, args = key.partition(":")
        with suppress(KeyError):
            return self[name](name, seperator, args, self, repoquery=repoquery)
        if fallback:
            with suppress(FormatterError):
                return fallback(name, seperator, args, self, repoquery=repoquery)
        raise FormatterError(f"{key!r} is not a valid formatter")

    def __getitem__(self, key: str) -> type[Formatter]:
        return self.__data[key]

    def __len__(self) -> int:
        return len(self.__data)

    def __iter__(self) -> Iterator[str]:
        return iter(self.__data)

    def new(
        self,
        other: Mapping[str, Callable | type[Formatter] | str],
        fallback: type[Formatter] | None | ellipsis = ...,
    ) -> Formatters:
        if fallback is not ...:
            pass
        elif isinstance(other, Formatters):
            fallback = other.fallback
        else:
            fallback = self.fallback
        return type(self)({**self, **other}, fallback)

    __or__ = new

    def with_fallback(self, fallback: type[Formatter] | None) -> Self:
        return type(self)(self.__data, fallback)

    def without(
        self,
        names: Collection[str] = frozenset(),
        formatters: Collection[type[Formatter]] = frozenset(),
    ) -> Self:
        newd = {k: v for k, v in self.items() if k not in names and v not in formatters}
        return type(self)(newd, self.fallback)

    def __contains__(self, name: object) -> bool:
        if not isinstance(name, str):
            raise TypeError
        try:
            self.get_formatter(name)
        except FormatterError:
            return False
        else:
            return True

    def singleline(self) -> Formatters:
        return Formatters(
            {
                name: formatter
                for name, formatter in self.items()
                if not formatter.MULTILINE
            }
        )

    def __repr__(self) -> str:
        return f"{type(self).__name__}({self.__data!r})"

    def __hash__(self) -> int:
        return hash(frozenset(self.items()))

    def _formattersv(self, formatters) -> dict[str, type[Formatter]]:
        if not formatters:
            return {}
        for name, formatter in formatters.items():
            if isinstance(formatter, type) and issubclass(formatter, Formatter):
                pass
            elif isinstance(formatter, str):
                formatters[name] = Formatter.frompat(formatter, name)
            elif callable(formatter):
                formatters[name] = Formatter.fromfunc(
                    formatter,  # type: ignore[arg-type]
                    name,
                )
            else:
                raise TypeError
        return formatters

    def _argcompleter(
        self,
        *,
        prefix: str,  # noqa: ARG002
        action: argparse.Action,  # noqa: ARG002
        parser: argparse.ArgumentParser,  # noqa: ARG002
        parsed_args: argparse.Namespace,  # noqa: ARG002
    ) -> list[str]:
        return sorted(self.formatters_it())

    def formatters_it(
        self,
        attrs: bool = True,
        formatters: bool = True,
        special_formatters: bool = True,
    ) -> Iterator[str]:
        """
        Yields:
            Names of formatters in this container. `SpecialFormatter`s have a
            `:` appended to their names.
        """
        if attrs:
            yield from _ATTRS
        if {formatters, special_formatters} == {False}:
            return
        for name, formatter in self.items():
            if issubclass(formatter, SpecialFormatter):
                if special_formatters:
                    yield f"{name}:"
            elif formatters:
                yield name


class SourceFromatter(Formatter):
    def format(self, packages: Iterable[PackageCompat]) -> Iterable[str]:
        return sorted(set(map(self.format_line, packages)))

    def format_line(self, package: PackageCompat) -> str:
        return get_source_name(package)


class SpecialFormatter(Formatter):
    """
    Formatter that accepts arguments
    """

    ATTRS: tuple[str, ...] = Formatter.ATTRS

    def _get_attrs(
        self, args: str | list[str], allow_multiline: bool = False
    ) -> Iterator[str | Formatter]:
        # Don't use the fallback formatter when looking up attributes.
        # We treat attributes (from ATTRS) that would normally be handled by fallback
        attrs = self.container.with_fallback(None)
        if not allow_multiline:
            attrs = attrs.singleline().new({"attr": _AttrFormatterSingleLine})
        for attr in args.split(",") if isinstance(args, str) else args:
            if attr in self.ATTRS:
                yield attr
            elif attr in attrs:
                yield attrs.get_formatter(attr, repoquery=self.rq)
            elif attr in _MULTILINE_ATTRS:
                self.err(
                    f"Attribute {attr!r} contains multiple lines"
                    f" which is not allowed by {self.name} formatter"
                )
            else:
                self.err(f"invalid argument {attr!r}")

    def validate(self) -> None:
        if not self.args.strip() or self.args.strip() == ",":
            self.err("received less than 1 argument")


class QueryFormatFormatter(SpecialFormatter):
    """
    Formatter that accepts dnf-style queryformat string
    """

    _REGEX = re.compile(r"(?P<leading>%?)%{(?P<macro>[^}]*)}")
    _INVALID_REGEX = re.compile(r"%{[^}]*$")
    ATTRS = _ATTRS_SINGLE
    MULTILINE = False

    def validate(self) -> None:
        if self._INVALID_REGEX.search(self.args):
            self.err("Unterminated query format string")
        # Don't allow nested queryformat strings
        self.container = self.container.without(formatters={type(self)})
        self._matches = list(self._REGEX.finditer(self.args))
        self._attrs = {
            macro: next(self._get_attrs([macro], False))
            for macro in {
                macro.group("macro")
                for macro in self._matches
                if not macro.group("leading")
            }
        }

    def format_line(self, package: PackageCompat) -> str:
        def fmt_part(
            full_string: str, match_obj: re.Match, package: PackageCompat
        ) -> str:
            groups = match_obj.groupdict()
            leading = groups["leading"]
            macro = groups["macro"]
            if leading:
                return full_string[match_obj.end("leading") :]
            attr = self._attrs[macro]
            if isinstance(attr, str):
                return _stringify(
                    getattr(package, attr),
                    multiline_allowed=False,
                )
            return attr.format_line(package)

        string = self.args
        matches = self._matches
        if not matches:
            return string
        parts: list[str] = []
        for idx, match_obj in enumerate(matches):
            a, z = match_obj.span()
            if idx == 0:
                parts.append(string[:a])
            parts.append(fmt_part(string, match_obj, package))
            if idx + 1 < len(matches):
                parts.append(string[z : matches[idx + 1].start()])
            else:
                parts.append(string[z:])
        return "".join(parts)


class AttrFormatter(SpecialFormatter):
    MULTILINE = True
    _MULTILINE_SEPERATOR = _DEFAULT_MULTILINE_SEPERATOR
    _MULTILINE_ALLOWED = True

    def validate(self) -> None:
        super().validate()
        self._validate()

    def _validate(self) -> None:
        if self.args not in self.ATTRS:
            raise FormatterError(f"'{self.args}' is not a valid attribute")
        self.attr = self.args

    def format_line(self, package: PackageCompat) -> str:
        # Return one string if there's multiple lines
        return "\n".join(self.format([package]))

    def format(self, packages: Iterable[PackageCompat]) -> Iterable[str]:
        for p in sorted(packages):
            result = getattr(p, self.attr)
            if isinstance(result, Iterable) and not isinstance(result, str):
                yield from map(
                    partial(
                        _stringify,
                        multiline_allowed=self._MULTILINE_ALLOWED,
                        multiline_seperator=self._MULTILINE_SEPERATOR,
                    ),
                    result,
                )
            else:
                yield _stringify(
                    result,
                    multiline_allowed=self._MULTILINE_ALLOWED,
                    multiline_seperator=self._MULTILINE_SEPERATOR,
                )


class _AttrFormatterSingleLine(AttrFormatter):
    ATTRS = _ATTRS_SINGLE
    _MULTILINE_ALLOWED = False


class AttrFallbackFormatter(AttrFormatter):
    def validate(self) -> None:
        if self.seperator:
            self.err("no arguments are accepted")
        self.args = self.name
        self._validate()


class _AttrFallbackFormatterUnseperated(AttrFallbackFormatter):
    _MULTILINE_SEPERATOR = ""


class JsonFormatter(SpecialFormatter):
    MULTILINE = True

    def validate(self) -> None:
        super().validate()
        self.attrs: list[Formatter | str] = list(self._get_attrs(self.args))

    def _format(self, package: PackageCompat) -> Iterable[tuple[str, Any]]:
        for attr in self.attrs:
            if isinstance(attr, str):
                result = getattr(package, attr)
                if isinstance(result, Iterable) and not isinstance(result, str):
                    result = [str(i) for i in result]
            else:
                result = attr.format_line(package)
            yield str(attr), result

    def format(self, packages: Iterable[PackageCompat]):
        import json

        data = [dict(self._format(package)) for package in packages]
        yield json.dumps(data, indent=2)

    format_line = format_line_notimplemented


class SingleLineFormatter(SpecialFormatter):
    DEFAULT_DIVIDER = " : "
    ATTRS: tuple[str, ...] = _ATTRS_SINGLE
    _ALLOW_MULTILINE_ATTRS: bool = False

    def validate(self) -> None:
        args, seperator, args2 = self.args.rpartition(":")
        LOG.debug(
            "parsing %s args: %r, %r, %r",
            "SingleLineFormatter.validate",
            args,
            seperator,
            args2,
        )
        # line:name:| -> params=name, divider = |
        if args and seperator and args2:
            LOG.debug("case 1")
            self.params = args
            self.divider = args2
        # line:name -> params=name, divider = DEFAULT_DIVIDER
        # line: -> params='', divider=DEFAULT_DIVIDER -> error in _get_attr()
        # line:: -> params='', divider=DEFAULT_DIVIDER -> error in _get_attr()
        elif not args:
            LOG.debug("case 2")
            self.params = args2
            self.divider = self.DEFAULT_DIVIDER
        # line:name: (trailing :) -> params=name, divider=DEFAULT_DIVIDER
        # line:otherformatter:arg: -> params=otherformatter:args,
        #                             divider=DEFAULT_DIVIDER
        # line::: -> error
        elif args and seperator and not args2:
            LOG.debug("case 3")
            self.params = args
            self.divider = self.DEFAULT_DIVIDER
        else:
            raise AssertionError("unreachable")
        self.attrs = list(self._get_attrs(self.params, self._ALLOW_MULTILINE_ATTRS))

    def _fl(
        self,
        package: PackageCompat,
        index: int,
        attr: str | Formatter,
        divider: str,
        multiline_allowed=False,
    ) -> str:
        out = ""
        if isinstance(attr, str):
            out += _stringify(
                getattr(package, attr), multiline_allowed=multiline_allowed
            )
        else:
            attr.rq = self.rq
            out += attr.format_line(package)
        if index != len(self.attrs) - 1:
            out += divider
        return out

    def format_line(self, package: PackageCompat) -> str:
        out = ""
        for index, attr in enumerate(self.attrs):
            out += self._fl(package, index, attr, self.divider)
        return out


class MultilineFormatter(SingleLineFormatter):
    ATTRS = SpecialFormatter.ATTRS
    MULTILINE = True
    _ALLOW_MULTILINE_ATTRS = True

    format_line = format_line_notimplemented

    def validate(self) -> None:
        super().validate()
        if len(self.attrs) != 2:
            self.err("requires two arguments")

    def format(self, packages: Iterable[PackageCompat]) -> Iterator[str]:
        for package in sorted(packages):
            initial = self._fl(package, 0, self.attrs[0], self.divider, False)
            attr = self.attrs[1]
            call: Formatter = (
                self.container.get_formatter(
                    attr, fallback=_AttrFallbackFormatterUnseperated
                )
                if isinstance(attr, str)
                else attr
            )
            call.rq = self.rq
            for line in call.format([package]):
                sublines: list[str] = line.splitlines() if "\n" in line else [line]
                yield from (initial + subline for subline in sublines)


def remote_location(_, package: PackageCompat) -> str:
    return _stringify(package.remote_location())


class RequiresMatchFormatter(SpecialFormatter):
    format_line = format_line_notimplemented
    _WRSRC: bool = False
    MULTILINE = True

    def format(self, packages: Iterable[PackageCompat]) -> Iterator[str]:
        if not self.rq:
            raise TypeError("self.rq is not set")
        requires = {
            str(require) for package in packages for require in package.requires
        }
        match_names = self.args.split(";") if ";" in self.args else self.args.split(",")
        if self._WRSRC:
            src_packages = self.rq.resolve_pkg_specs(match_names).filterm(arch="src")
            match_packages = set(self.rq.get_subpackages(src_packages))
        else:
            match_packages = set(
                self.rq.resolve_pkg_specs(match_names, resolve=False, with_src=False)
            )
        for reldep in requires:
            if (
                set(self.rq.resolve_pkg_specs([reldep], resolve=True, with_src=False))
                & match_packages
            ):
                yield reldep


class RequiresMatchSrcFormatter(RequiresMatchFormatter):
    _WRSRC = True


class _RequiresMatchPrefixFormatter(RequiresMatchFormatter):
    PREFIX: ClassVar[str | staticmethod[[PackageCompat], str]]

    def format(self, packages: Iterable[PackageCompat]) -> Iterator[str]:
        for package in sorted(packages):
            prefix_fmt = self.PREFIX
            prefix = (
                prefix_fmt(package)
                if callable(prefix_fmt)
                else prefix_fmt.format(package=package)
            )
            yield from (prefix + o for o in super().format([package]))


class NARequiresMatchFormatter(_RequiresMatchPrefixFormatter):
    PREFIX = "{package.name}.{package.arch} : "


class SourceRequiresMatch(_RequiresMatchPrefixFormatter):

    @staticmethod
    def PREFIX(package: PackageCompat) -> str:  # type: ignore[override]
        return f"{get_source_name(package)} : "


class NARequiresMatchSrcFormatter(
    RequiresMatchSrcFormatter, NARequiresMatchFormatter
): ...


DefaultFormatters = Formatters(
    {
        "plain": "{0}",
        "plainwithrepo": "{0} {0.reponame}",
        "nevrr": "{0} {0.reponame}",
        "na": "{0.name}.{0.arch}",
        "nev": "{0.name}-{0.epoch}:{0.version}",
        "nevr": "{0.name}-{0.evr}",
        "nevra": "{0}",
        "full_nevra": "{0.name}-{0.evr}.{0.arch}",
        "nv": "{0.name}-{0.version}",
        "source": SourceFromatter,
        "src": SourceFromatter,
        "queryformat": QueryFormatFormatter,
        "qf": QueryFormatFormatter,
        "attr": AttrFormatter,
        "json": JsonFormatter,
        "line": SingleLineFormatter,
        "remote_location": remote_location,
        "multiline": MultilineFormatter,
        "requiresmatch": RequiresMatchFormatter,
        "rm": RequiresMatchFormatter,
        "source+requiresmatch": SourceRequiresMatch,
        "source+rm": SourceRequiresMatch,
        "na-requiresmatch": NARequiresMatchFormatter,
        "narm": NARequiresMatchFormatter,
        "requiresmatch-src": RequiresMatchSrcFormatter,
        "rmsrc": RequiresMatchSrcFormatter,
        "narmsrc": NARequiresMatchSrcFormatter,
        "na-requiresmatch-src": NARequiresMatchSrcFormatter,
    },
    AttrFallbackFormatter,
)



# Copyright (C) 2024 Maxwell G <maxwell@gtmx.me>
# SPDX-License-Identifier: GPL-2.0-or-later

from __future__ import annotations

import os
from functools import cache

from fedrq.backends import get_default_backend
from fedrq.backends.base import BackendMod


@cache
def ensure_default_backend() -> BackendMod:
    backend = os.environ.get("FEDRQ_BACKEND")
    gotten = get_default_backend(backend, bool(backend))
    # Check that get_default_backend does the right thing
    assert (backend or gotten.BACKEND) == gotten.BACKEND
    return gotten



"""
This type stub file was generated by pyright.
"""

if __package__ or "." in __name__:
    ...
else:
    ...

class _SwigNonDynamicMeta(type):
    """Meta class to enforce nondynamic attributes (no new attributes) for a class"""

    __setattr__ = ...

class Version:
    r"""Plugin version"""

    thisown = ...
    __repr__ = ...
    major = ...
    minor = ...
    micro = ...
    def __init__(self, major, minor, micro) -> None: ...

    __swig_destroy__ = ...

class IPlugin:
    r"""A base class for implementing LIBDNF5 plugins that introduce additional logic into the library using hooks."""

    thisown = ...
    __repr__ = ...
    def __init__(self, base) -> None: ...

    __swig_destroy__ = ...
    def get_api_version(self):
        r"""Returns the version of the API supported by the plugin. It can be called at any time."""
        ...

    def get_name(self):
        r"""Returns the name of the plugin. It can be called at any time."""
        ...

    def get_version(self):
        r"""Gets the plugin version. It can be called at any time."""
        ...

    def get_attributes(self):
        r"""
        :rtype: string
        :return: A nullptr terminated array of attributes supported by the plugin.
        """
        ...

    def get_attribute(self, name):
        r"""
        Gets the value of the attribute from the plugin. Returns nullptr if the attribute does not exist.
        It can be called at any time.
        """
        ...

    def load_plugins(self):
        r"""
        The plugin can load additional plugins. E.g. C++ plugin for loading Python plugins.
        Called before init.
        """
        ...

    def init(self):
        r"""
        Plugin initialization.
        Called before hooks.
        """
        ...

    def pre_base_setup(self):
        r"""
        The pre_base_setup hook.
        It is called at the beginning of the `Base::setup` method (after the `init` hook).
        """
        ...

    def post_base_setup(self):
        r"""
        The post_base_setup hook.
        It is called at the end of the `Base::setup` method.
        """
        ...

    def pre_transaction(self, arg0):
        r"""
        The pre_transaction hook.
        It is called just before the actual transaction starts.
        """
        ...

    def post_transaction(self, arg0):
        r"""
        The post_transaction hook.
        It is called after transactions.
        """
        ...

    def finish(self):
        r"""Finish the plugin and release all resources obtained by the init method and in hooks."""
        ...

    def get_base(self): ...
    def __disown__(self):  # -> Any:
        ...



# Copyright (C) 2024 Maxwell G <maxwell@gtmx.me>
# SPDX-License-Identifier: GPL-2.0-or-later

"""
This module (and the other experimental modules in fedrq) are not meant for public use.
They are subject to breaking changes in minor releases and should not be relied
on by external code.
Once the functionality has stabilized, this code will be moved out of the
experimental namespace.
"""

from __future__ import annotations

from collections.abc import Iterator
from enum import Enum
from typing import TYPE_CHECKING, Any, Protocol, TypedDict, TypeVar

if TYPE_CHECKING:
    from typing_extensions import Self, TypeAlias, Unpack

_T_co = TypeVar("_T_co", covariant=True)


class SimpleSequence(Protocol[_T_co]):
    """
    Represents a simplified version of the collections.abc.Sequence API that is
    fulfilled by the wrapped C++ vectors in the dnf5 API
    """

    def __len__(self) -> int: ...
    def __iter__(self) -> Iterator[_T_co]: ...
    def __getitem__(self, index: int, /) -> _T_co: ...


class GroupPackage(Protocol):
    @property
    def name(self) -> str: ...
    @property
    def package_type(self) -> int: ...
    def __lt__(self, other: Self) -> bool: ...
    def __gt__(self, other: Self) -> bool: ...
    def __le__(self, other: Self) -> bool: ...
    def __ge__(self, other: Self) -> bool: ...


class Group(Protocol):
    @property
    def groupid(self) -> str: ...
    @property
    def name(self) -> str: ...
    @property
    def description(self) -> str: ...
    @property
    def translated_name(self) -> str: ...
    @property
    def translated_description(self) -> str: ...
    @property
    def langonly(self) -> bool: ...
    @property
    def uservisible(self) -> bool: ...
    @property
    def packages(self) -> SimpleSequence[GroupPackage]: ...
    def get_packages_of_type(
        self, package_type: int, /
    ) -> SimpleSequence[GroupPackage]: ...
    def __lt__(self, other: Self) -> bool: ...
    def __gt__(self, other: Self) -> bool: ...
    def __le__(self, other: Self) -> bool: ...
    def __ge__(self, other: Self) -> bool: ...
    def __hash__(self) -> int: ...


_StrIter: TypeAlias = "list[str] | tuple[str, ...] | str"


class GroupQueryKwargs(TypedDict, total=False):
    groupid: _StrIter
    groupid__eq: _StrIter
    groupid__neq: _StrIter
    groupid__glob: _StrIter
    groupid__contains: _StrIter

    name: _StrIter
    name__eq: _StrIter
    name__neq: _StrIter
    name__glob: _StrIter
    name__contains: _StrIter

    package_name: _StrIter
    package_name__eq: _StrIter
    package_name__neq: _StrIter
    package_name__glob: _StrIter
    package_name__contains: _StrIter

    uservisible: bool

    langonly: bool


class GroupQuery(Protocol):
    def filterm(self, **kwargs: Unpack[GroupQueryKwargs]) -> Self: ...
    def filter_groupid(self, patterns: _StrIter, cmp: int = ..., /) -> None: ...
    def filter_name(self, patterns: _StrIter, cmp: int = ..., /) -> None: ...
    def filter_package_name(self, patterns: _StrIter, cmp: int = ..., /) -> None: ...
    def filter_uservisible(self, value: bool) -> None: ...

    # Not implemented in libdnf5
    # def filter_langonly(self, value: bool) -> None: ...

    # Set methods
    def difference(self, other: Self) -> Self:
        """
        Set difference.
        Depending on the backend, this may change 'self' in place or return a
        new GroupQuery
        """
        ...

    def union(self, other: Self) -> Self:
        """
        Set union.
        Depending on the backend, this may change 'self' in place or return a
        new GroupQuery.
        """
        ...

    def intersection(self, other: Self) -> Self:
        """
        Set intersection.
        Depending on the backend, this may change 'self' in place or return a
        new GroupQuery.
        """
        ...

    def update(self, other: Self) -> None:
        """
        Update set in-place
        """
        ...

    def add(self, other: Group, /) -> None: ...

    def clear(self) -> None: ...

    def __bool__(self) -> bool: ...

    def remove(self, other: Group, /) -> None: ...

    def __contains__(self, other) -> bool: ...
    def __len__(self) -> int: ...
    def __iter__(self) -> Iterator[Group]: ...
    def __ior__(self, other) -> Self: ...
    def __iand__(self, other) -> Self: ...
    def __isub__(self, other) -> Self: ...


class Environment(Protocol):
    @property
    def environmentid(self) -> str: ...
    @property
    def name(self) -> str: ...
    @property
    def description(self) -> str: ...
    @property
    def translated_name(self) -> str: ...
    @property
    def translated_description(self) -> str: ...

    # TODO: libdnf5 returns strings, not Group objects, so we're not exposing
    # this for now
    # @property
    # def groups(self) -> SimpleSequence[Group]: ...
    # @property
    # def optional_groups(self) -> SimpleSequence[Group]: ...

    @property
    def groupids(self) -> SimpleSequence[str]: ...
    @property
    def optional_groupids(self) -> SimpleSequence[str]: ...

    def __lt__(self, other: Self) -> bool: ...
    def __gt__(self, other: Self) -> bool: ...
    def __le__(self, other: Self) -> bool: ...
    def __ge__(self, other: Self) -> bool: ...
    def __hash__(self) -> int: ...


class EnvironmentQueryKwargs(TypedDict, total=False):
    environmentid: _StrIter
    environmentid__eq: _StrIter
    environmentid__neq: _StrIter
    environmentid__glob: _StrIter
    environmentid__contains: _StrIter

    name: _StrIter
    name__eq: _StrIter
    name__neq: _StrIter
    name__glob: _StrIter
    name__contains: _StrIter


class EnvironmentQuery(Protocol):
    def filterm(self, **kwargs: Unpack[EnvironmentQueryKwargs]) -> Self: ...
    def filter_environmentid(self, patterns: _StrIter, cmp: int = ..., /) -> None: ...
    def filter_name(self, patterns: _StrIter, cmp: int = ..., /) -> None: ...

    # Set methods
    def difference(self, other: Self) -> Self:
        """
        Set difference.
        Depending on the backend, this may change 'self' in place or return a
        new GroupQuery
        """
        ...

    def union(self, other: Self) -> Self:
        """
        Set union.
        Depending on the backend, this may change 'self' in place or return a
        new GroupQuery.
        """
        ...

    def intersection(self, other: Self) -> Self:
        """
        Set intersection.
        Depending on the backend, this may change 'self' in place or return a
        new GroupQuery.
        """
        ...

    def update(self, other: Self) -> None:
        """
        Update set in-place
        """
        ...

    def add(self, other: Environment, /) -> None: ...

    def clear(self) -> None: ...

    def __bool__(self) -> bool: ...

    def remove(self, other: Environment, /) -> None: ...

    def __contains__(self, other) -> bool: ...
    def __len__(self) -> int: ...
    def __iter__(self) -> Iterator[Environment]: ...
    def __ior__(self, other: Self) -> Self: ...
    def __iand__(self, other: Self) -> Self: ...
    def __isub__(self, other: Self) -> Self: ...


def _filter_func(obj: Any, query_cmp_enum: type[Enum], **kwargs: Any):
    cmp_mapping = {member.name.lower(): member.value for member in query_cmp_enum}
    for key, value in kwargs.items():
        func, sep, cmp = key.partition("__")
        if sep and not cmp or cmp and cmp not in cmp_mapping:
            raise ValueError(f"Invalid key: {key}")
        filter_func = getattr(obj, f"filter_{func}")
        if cmp:
            cmp_member = cmp_mapping[cmp]
            filter_func(value, cmp_member)
        else:
            filter_func(value)



"""
This type stub file was generated by pyright.
"""

if __package__ or "." in __name__:
    ...
else:
    ...

class _SwigNonDynamicMeta(type):
    """Meta class to enforce nondynamic attributes (no new attributes) for a class"""

    __setattr__ = ...

class SwigPyIterator:
    thisown = ...
    def __init__(self, *args, **kwargs) -> None: ...

    __repr__ = ...
    __swig_destroy__ = ...
    def value(self): ...
    def incr(self, n=...): ...
    def decr(self, n=...): ...
    def distance(self, x): ...
    def equal(self, x): ...
    def copy(self): ...
    def next(self): ...
    def __next__(self): ...
    def previous(self): ...
    def advance(self, n): ...
    def __eq__(self, x) -> bool: ...
    def __ne__(self, x) -> bool: ...
    def __iadd__(self, n): ...
    def __isub__(self, n): ...
    def __add__(self, n): ...
    def __sub__(self, *args): ...
    def __iter__(self):  # -> Self:
        ...

class AdvisoryId:
    thisown = ...
    __repr__ = ...
    def __init__(self, *args) -> None: ...
    def __eq__(self, other) -> bool: ...
    def __ne__(self, other) -> bool: ...

    id = ...
    __swig_destroy__ = ...

class Advisory:
    r"""An advisory, represents advisory used to track security updates"""

    thisown = ...
    def __init__(self, *args, **kwargs) -> None: ...

    __repr__ = ...
    def __eq__(self, other) -> bool: ...
    def __ne__(self, other) -> bool: ...

    __swig_destroy__ = ...
    def get_name(self):
        r"""
        Get name of this advisory.

        :rtype: string
        :return: Name of this advisory as std::string.
        """
        ...

    def get_severity(self):
        r"""
        Get severity of this advisory.

        :rtype: string
        :return: Severity of this advisory as std::string.
        """
        ...

    def get_type(self):
        r"""
        Get type of this advisory.
        Possible types are: "security", "bugfix", "enhancement", "newpackage".

        :rtype: string
        :return: type of this advisory as std::string.
        """
        ...

    def get_buildtime(self):
        r"""
        Get buildtime of this advisory. Libsolv combines issued and updated dates
        into buildtime by always using the newer one.

        :rtype: int
        :return: buildtime of this advisory.
        """
        ...

    def get_vendor(self):
        r"""
        Get vendor of this advisory.

        :rtype: string
        :return: Vendor of this advisory as std::string.
        """
        ...

    def get_description(self):
        r"""
        Get description of this advisory.

        :rtype: string
        :return: Description of this advisory as std::string.
        """
        ...

    def get_title(self):
        r"""
        Get title of this advisory.

        :rtype: string
        :return: Title of this advisory as std::string.
        """
        ...

    def get_status(self):
        r"""
        Get status of this advisory.

        :rtype: string
        :return: Status of this advisory as std::string.
        """
        ...

    def get_rights(self):
        r"""
        Get rights of this advisory.

        :rtype: string
        :return: Rights of this advisory as std::string.
        """
        ...

    def get_message(self):
        r"""
        Get message of this advisory.

        :rtype: string
        :return: Message of this advisory as std::string.
        """
        ...

    def get_id(self):
        r"""
        Get AdvisoryId.

        :rtype: :py:class:`AdvisoryId`
        :return: AdvisoryId of this advisory.
        """
        ...

    def get_references(self, *args):
        r"""
        Get all references of specified type from this advisory.
        Possible refenrece types are: "bugzilla", "cve", "vendor".

        :type types: std::vector< std::string,std::allocator< std::string > >, optional
        :param types:     What types of references to get. If not specified gets all types.
        :rtype: std::vector< libdnf5::advisory::AdvisoryReference,std::allocator< libdnf5::advisory::AdvisoryReference > >
        :return: Vector of AdvisoryReference objects.
        """
        ...

    def get_collections(self):
        r"""
        Get all collections from this advisory.

        :rtype: std::vector< libdnf5::advisory::AdvisoryCollection,std::allocator< libdnf5::advisory::AdvisoryCollection > >
        :return: Vector of AdvisoryCollection objects.
        """
        ...

    def is_applicable(self):
        r"""
        Check whether at least one collection from this advisory is applicable.

        :rtype: boolean
        :return: True if applicable, False otherwise.
        """
        ...

class AdvisoryPackage:
    thisown = ...
    __repr__ = ...
    def __init__(self, *args) -> None: ...

    __swig_destroy__ = ...
    def get_name(self):
        r"""
        Get name of this AdvisoryPackage.

        :rtype: string
        :return: Name of this AdvisoryPackage as std::string.
        """
        ...

    def get_epoch(self):
        r"""
        Get epoch of this AdvisoryPackage.

        :rtype: string
        :return: Epoch of this AdvisoryPackage as std::string.
        """
        ...

    def get_version(self):
        r"""
        Get version of this AdvisoryPackage.

        :rtype: string
        :return: Version of this AdvisoryPackage as std::string.
        """
        ...

    def get_release(self):
        r"""
        Get release version of this AdvisoryPackage.

        :rtype: string
        :return: Release of this AdvisoryPackage as std::string.
        """
        ...

    def get_evr(self):
        r"""
        Get evr of this AdvisoryPackage.

        :rtype: string
        :return: Evr of this AdvisoryPackage as std::string.
        """
        ...

    def get_arch(self):
        r"""
        Get arch of this AdvisoryPackage.

        :rtype: string
        :return: Arch of this AdvisoryPackage as std::string.
        """
        ...

    def get_nevra(self):
        r"""
        Get NEVRA of this AdvisoryPackage.

        :rtype: string
        :return: NEVRA of this AdvisoryPackage as std::string.
        """
        ...

    def get_advisory_id(self):
        r"""
        Get AdvisoryId of Advisory this AdvisoryPackage belongs to.

        :rtype: :py:class:`AdvisoryId`
        :return: AdvisoryId of this AdvisoryPackage.
        """
        ...

    def get_reboot_suggested(self):
        r"""
        Get bool value whether reboot is suggested

        :rtype: boolean
        :return: Reboot suggestion of this advisory package as bool
        """
        ...

    def get_restart_suggested(self):
        r"""
        Get bool value whether restart is suggested

        :rtype: boolean
        :return: Restart suggestion of this advisory package as bool
        """
        ...

    def get_relogin_suggested(self):
        r"""
        Get bool value whether relogin is suggested

        :rtype: boolean
        :return: Relogin suggestion of this advisory package as bool
        """
        ...

    def get_advisory(self):
        r"""
        Get Advisory this AdvisoryPackage belongs to.

        :rtype: :py:class:`Advisory`
        :return: newly construted Advisory object of this AdvisoryPackage.
        """
        ...

    def get_advisory_collection(self):
        r"""
        Get AdvisoryCollection this AdvisoryPackage belongs to.

        :rtype: :py:class:`AdvisoryCollection`
        :return: newly construted AdvisoryCollection object of this AdvisoryPackage.
        """
        ...

class AdvisorySet:
    thisown = ...
    __repr__ = ...
    def __init__(self, *args) -> None: ...

    __swig_destroy__ = ...
    def begin(self): ...
    def end(self): ...
    def __ior__(self, other): ...
    def __isub__(self, other): ...
    def __iand__(self, other): ...
    def update(self, other):
        r"""
        Set union: elements that are in the current set or in the `other` set.

        :type other: :py:class:`AdvisorySet`
        :param other: The set to unify with.
        :raises: UsedDifferentSack When the sets entering the operation do not share the same AdvisorySack.
        Since: 5.0
        """
        ...

    def intersection(self, other):
        r"""
        Set intersection: elements in the current set that are also in the `other` set.

        :type other: :py:class:`AdvisorySet`
        :param other: The set to intersect with.
        :raises: UsedDifferentSack When the sets entering the operation do not share the same AdvisorySack.
        Since: 5.0
        """
        ...

    def difference(self, other):
        r"""
        Set difference: elements in the current set that are not in the `other` set.

        :type other: :py:class:`AdvisorySet`
        :param other: The set to check for differences in.
        :raises: UsedDifferentSack When the sets entering the operation do not share the same AdvisorySack.
        Since: 5.0
        """
        ...

    def clear(self):
        r"""
        Remove all advisories from the set.

        Since: 5.0
        """
        ...

    def empty(self):
        r"""
        :rtype: boolean
        :return: `true` if the set is empty, `false` otherwise.
        Since: 5.0
        """
        ...

    def add(self, adv):
        r"""
        Add `adv` to the set.

        :type adv: :py:class:`Advisory`
        :param adv: Advisory to be added to the set.
        Since: 5.0
        """
        ...

    def contains(self, adv):
        r"""
        :rtype: boolean
        :return: `true` if an advisory is in the set, `false` otherwise.
        :type adv: :py:class:`Advisory`
        :param adv: Advisory that is tested for presence.
        Since: 5.0
        """
        ...

    def remove(self, adv):
        r"""
        Remove `adv` from the set.

        :type adv: :py:class:`Advisory`
        :param adv: Advisory to be removed from the set.
        Since: 5.0
        """
        ...

    def get_base(self): ...
    def size(self):
        r"""
        :rtype: int
        :return: Number of elements in the set.
        """
        ...

    def swap(self, other): ...
    def get_advisory_packages_sorted_by_name_arch_evr(self, only_applicable=...):
        r"""
        Gather AdvisoryPackages for each Advisory in the set.
        The AdvisoryPackages are sorted by libsolv `id`s of name, arch and evr.
        This is a different sorting than sorting by the strings of AdvisoryPackages names, architectures and evrs.

        :type only_applicable: boolean, optional
        :param only_applicable: Whether to return only AdvisoryPackages from applicable AdvisoryCollections.
        Since: 5.0
        """
        ...

class AdvisorySetIterator:
    thisown = ...
    __repr__ = ...
    def __init__(self, other) -> None: ...

    __swig_destroy__ = ...
    def value(self): ...
    def next(self): ...
    def __eq__(self, other) -> bool: ...
    def __ne__(self, other) -> bool: ...
    def begin(self, *args): ...
    def end(self, *args): ...

class AdvisoryModule:
    thisown = ...
    __repr__ = ...
    def __init__(self, *args) -> None: ...

    __swig_destroy__ = ...
    def get_name(self):
        r"""
        Get name of this AdvisoryModule.

        :rtype: string
        :return: Name of this AdvisoryModule as std::string.
        """
        ...

    def get_stream(self):
        r"""
        Get stream of this AdvisoryModule.

        :rtype: string
        :return: Stream of this AdvisoryModule as std::string.
        """
        ...

    def get_version(self):
        r"""
        Get version of this AdvisoryModule.

        :rtype: string
        :return: Version of this AdvisoryModule as std::string.
        """
        ...

    def get_context(self):
        r"""
        Get context of this AdvisoryModule.

        :rtype: string
        :return: Context of this AdvisoryModule as std::string.
        """
        ...

    def get_arch(self):
        r"""
        Get arch of this AdvisoryModule.

        :rtype: string
        :return: Arch of this AdvisoryModule as std::string.
        """
        ...

    def get_nsvca(self):
        r"""
        Get NSVCA of this AdvisoryModule.

        :rtype: string
        :return: NSVCA of this AdvisoryModule as std::string.
        """
        ...

    def get_advisory_id(self):
        r"""
        Get AdvisoryId of Advisory this AdvisoryModule belongs to.

        :rtype: :py:class:`AdvisoryId`
        :return: AdvisoryId of this AdvisoryModule.
        """
        ...

    def get_advisory(self):
        r"""
        Get Advisory this AdvisoryModule belongs to.

        :rtype: :py:class:`Advisory`
        :return: newly construted Advisory object of this AdvisoryModule.
        """
        ...

    def get_advisory_collection(self):
        r"""
        Get AdvisoryCollection this AdvisoryModule belongs to.

        :rtype: :py:class:`AdvisoryCollection`
        :return: newly construted AdvisoryCollection object of this AdvisoryModule.
        """
        ...

class AdvisoryCollection:
    thisown = ...
    def __init__(self, *args, **kwargs) -> None: ...

    __repr__ = ...
    def is_applicable(self):
        r"""
        Whether this AdvisoryCollection is applicable. True when at least one AdvisoryModule in this
        AdvisoryCollection is active on the system, False otherwise.
        """
        ...

    def get_advisory_id(self):
        r"""
        Get AdvisoryId of Advisory this AdvisoryCollection belongs to.

        :rtype: :py:class:`AdvisoryId`
        :return: AdvisoryId of this AdvisoryCollection.
        """
        ...

    def get_packages(self):
        r"""
        Get all AdvisoryPackages stored in this AdvisoryCollection

        :rtype: std::vector< libdnf5::advisory::AdvisoryPackage,std::allocator< libdnf5::advisory::AdvisoryPackage > >
        :return: std::vector of AdvisorPackages used as output.
        """
        ...

    def get_modules(self):
        r"""
        Get all AdvisoryModules stored in this AdvisoryCollection

        :rtype: std::vector< libdnf5::advisory::AdvisoryModule,std::allocator< libdnf5::advisory::AdvisoryModule > >
        :return: std::vector of AdvisorModules.
        """
        ...

    def get_advisory(self):
        r"""
        Get Advisory this AdvisoryCollection belongs to.

        :rtype: :py:class:`Advisory`
        :return: newly construted Advisory object of this AdvisoryCollection.
        """
        ...
    __swig_destroy__ = ...

class AdvisoryQuery(AdvisorySet):
    r"""
    AdvisoryQuery is the only way how to access advisories.
    It is constructed using Base and filled with advisories from enabled repositories in its RepoSack.
    """

    thisown = ...
    __repr__ = ...
    def __init__(self, *args) -> None: ...

    __swig_destroy__ = ...
    def filter_name(self, *args): ...
    def filter_type(self, *args): ...
    def filter_reference(self, *args): ...
    def filter_severity(self, *args): ...
    def filter_packages(self, *args):
        r"""
        Filter out advisories that don't contain at least one AdvisoryPackage that has a counterpart Package in package_set
        such that they have matching name and architecture and also their epoch-version-release complies to cmp_type.

        :type package_set: libdnf5::rpm::PackageSet
        :param package_set:  libdnf5::rpm::PackageSet used when filtering.
        :type cmp_type: int, optional
        :param cmp_type:     Condition to fulfill when comparing epoch-version-release of packages.
        """
        ...

    def get_advisory_packages_sorted(self, *args):
        r"""
        Get std::vector of AdvisoryPackages present in advisories from query.
        Each AdvisoryPackage is returned only if it has a counterpart Package in package_set such that they have matching
        name and architecture and also their epoch-version-release complies to cmp_type.
        AdvisoryPackages are sorted in the std::vector by Name, Arch and EVR.

        :type package_set: libdnf5::rpm::PackageSet
        :param package_set:  libdnf5::rpm::PackageSet used when filtering.
        :type cmp_type: int, optional
        :param cmp_type:     Condition to fulfill when comparing epoch-version-release of packages.
        :rtype: std::vector< libdnf5::advisory::AdvisoryPackage,std::allocator< libdnf5::advisory::AdvisoryPackage > >
        :return: std::vector of AdvisoryPackages
        """
        ...

class AdvisoryReference:
    thisown = ...
    def __init__(self, *args, **kwargs) -> None: ...

    __repr__ = ...
    def get_id(self):
        r"""
        Get id of this advisory reference, this id is like a name of this reference
        (it is not libsolv id).

        :rtype: string
        :return: id of this reference as std::string.
        """
        ...

    def get_type(self):
        r"""
        Get type of this reference.
        Possible reference types are: "bugzilla", "cve", "vendor".

        :rtype: string
        :return: type of this reference as std::string.
        """
        ...

    def get_type_cstring(self):
        r"""
        Get type of this reference.

        :rtype: string
        :return: Type of this reference as const char* !! (temporal value)
        """
        ...

    def get_title(self):
        r"""
        Get title of this reference.

        :rtype: string
        :return: Title of this reference.
        """
        ...

    def get_url(self):
        r"""
        Get url of this reference.

        :rtype: string
        :return: Url of this reference.
        """
        ...
    __swig_destroy__ = ...

class VectorAdvisoryModule:
    thisown = ...
    __repr__ = ...
    def iterator(self): ...
    def __iter__(self): ...
    def __nonzero__(self): ...
    def __bool__(self): ...
    def __len__(self): ...
    def __getslice__(self, i, j): ...
    def __setslice__(self, *args): ...
    def __delslice__(self, i, j): ...
    def __delitem__(self, *args): ...
    def __getitem__(self, *args): ...
    def __setitem__(self, *args): ...
    def pop(self): ...
    def append(self, x): ...
    def empty(self): ...
    def size(self): ...
    def swap(self, v): ...
    def begin(self): ...
    def end(self): ...
    def rbegin(self): ...
    def rend(self): ...
    def clear(self): ...
    def get_allocator(self): ...
    def pop_back(self): ...
    def erase(self, *args): ...
    def __init__(self, *args) -> None: ...
    def push_back(self, x): ...
    def front(self): ...
    def back(self): ...
    def assign(self, n, x): ...
    def insert(self, *args): ...
    def reserve(self, n): ...
    def capacity(self): ...

    __swig_destroy__ = ...

class VectorAdvisoryCollection:
    thisown = ...
    __repr__ = ...
    def iterator(self): ...
    def __iter__(self): ...
    def __nonzero__(self): ...
    def __bool__(self): ...
    def __len__(self): ...
    def __getslice__(self, i, j): ...
    def __setslice__(self, *args): ...
    def __delslice__(self, i, j): ...
    def __delitem__(self, *args): ...
    def __getitem__(self, *args): ...
    def __setitem__(self, *args): ...
    def pop(self): ...
    def append(self, x): ...
    def empty(self): ...
    def size(self): ...
    def swap(self, v): ...
    def begin(self): ...
    def end(self): ...
    def rbegin(self): ...
    def rend(self): ...
    def clear(self): ...
    def get_allocator(self): ...
    def pop_back(self): ...
    def erase(self, *args): ...
    def __init__(self, *args) -> None: ...
    def push_back(self, x): ...
    def front(self): ...
    def back(self): ...
    def assign(self, n, x): ...
    def insert(self, *args): ...
    def reserve(self, n): ...
    def capacity(self): ...

    __swig_destroy__ = ...

class VectorAdvisoryPackage:
    thisown = ...
    __repr__ = ...
    def iterator(self): ...
    def __iter__(self): ...
    def __nonzero__(self): ...
    def __bool__(self): ...
    def __len__(self): ...
    def __getslice__(self, i, j): ...
    def __setslice__(self, *args): ...
    def __delslice__(self, i, j): ...
    def __delitem__(self, *args): ...
    def __getitem__(self, *args): ...
    def __setitem__(self, *args): ...
    def pop(self): ...
    def append(self, x): ...
    def empty(self): ...
    def size(self): ...
    def swap(self, v): ...
    def begin(self): ...
    def end(self): ...
    def rbegin(self): ...
    def rend(self): ...
    def clear(self): ...
    def get_allocator(self): ...
    def pop_back(self): ...
    def erase(self, *args): ...
    def __init__(self, *args) -> None: ...
    def push_back(self, x): ...
    def front(self): ...
    def back(self): ...
    def assign(self, n, x): ...
    def insert(self, *args): ...
    def reserve(self, n): ...
    def capacity(self): ...

    __swig_destroy__ = ...

class VectorAdvisoryReference:
    thisown = ...
    __repr__ = ...
    def iterator(self): ...
    def __iter__(self): ...
    def __nonzero__(self): ...
    def __bool__(self): ...
    def __len__(self): ...
    def __getslice__(self, i, j): ...
    def __setslice__(self, *args): ...
    def __delslice__(self, i, j): ...
    def __delitem__(self, *args): ...
    def __getitem__(self, *args): ...
    def __setitem__(self, *args): ...
    def pop(self): ...
    def append(self, x): ...
    def empty(self): ...
    def size(self): ...
    def swap(self, v): ...
    def begin(self): ...
    def end(self): ...
    def rbegin(self): ...
    def rend(self): ...
    def clear(self): ...
    def get_allocator(self): ...
    def pop_back(self): ...
    def erase(self, *args): ...
    def __init__(self, *args) -> None: ...
    def push_back(self, x): ...
    def front(self): ...
    def back(self): ...
    def assign(self, n, x): ...
    def insert(self, *args): ...
    def reserve(self, n): ...
    def capacity(self): ...

    __swig_destroy__ = ...

def AdvisorySet__iter__(self):  # -> Iterator:
    ...



# Copyright (C) 2024 Maxwell G <maxwell@gtmx.me>
# SPDX-License-Identifier: GPL-2.0-or-later

"""
Tests for experimental comps code in backends
"""

from __future__ import annotations

from collections.abc import Callable
from typing import Any

import pytest

from fedrq.backends.base import BackendMod
from fedrq.backends.base.experimental import EnvironmentQuery, GroupQuery
from fedrq.config import get_config


def default_backend_experimental(default_backend: BackendMod):
    if default_backend.BACKEND == "dnf":
        from fedrq.backends.dnf.backend import experimental
    elif default_backend.BACKEND == "libdnf5":
        from fedrq.backends.libdnf5.backend import (  # type: ignore[no-redef]
            experimental,
        )
    else:
        raise ValueError
    return experimental


@pytest.fixture(scope="module")
def f39_base() -> Any:
    return get_config().get_rq("f39").base


@pytest.fixture
def new_group_query(
    default_backend: BackendMod, f39_base: Any
) -> Callable[..., GroupQuery]:
    experimental = default_backend_experimental(default_backend)
    return lambda: experimental.GroupQuery(f39_base)


@pytest.fixture
def new_env_query(
    default_backend: BackendMod, f39_base: Any
) -> Callable[..., EnvironmentQuery]:
    experimental = default_backend_experimental(default_backend)
    return lambda: experimental.EnvironmentQuery(f39_base)


@pytest.mark.no_rpm_mock
def test_group_query(new_group_query: Callable[..., GroupQuery]) -> None:
    group_query = new_group_query()
    group_query_new = group_query.filterm(name="Buildsystem building group")
    # Filtering is done is place
    assert group_query_new is group_query
    assert len(group_query) == 1
    group = next(iter(group_query))
    group_query = new_group_query()
    group_query.filter_groupid("buildsys-build")
    group_query.filterm(groupid__eq="buildsys-build")
    assert len(group_query) == 1
    assert next(iter(group_query)) == group
    group_query.filterm(groupid__neq="buildsys-build")
    assert len(group_query) == 0


@pytest.mark.no_rpm_mock
def test_group_query_filters(
    default_backend: BackendMod, new_group_query: Callable[..., GroupQuery]
) -> None:
    exp = default_backend_experimental(default_backend)
    group_query = new_group_query()
    group_query.filter_package_name(["rpmlint", "podman"])
    group_list = sorted(group_query)
    assert [group.groupid for group in group_list] == [
        "container-management",
        "rpm-development-tools",
    ]
    assert group_list[0] < group_list[1]
    assert group_list[0] <= group_list[1]
    assert group_list[0] == group_list[0]
    assert group_list[0] <= group_list[0]
    assert group_list[0] != group_list[1]
    assert group_list[1] > group_list[0]
    assert group_list[1] >= group_list[0]

    assert len(group_query) == 2
    group_query.remove(group_list[0])
    assert len(group_query) == 1
    assert group_query
    group_query.clear()
    assert len(group_query) == 0
    assert not group_query

    group_query2 = new_group_query()
    group_query2.filter_uservisible(False)
    assert "GNOME" in {group.name for group in group_query2}

    # Test set operations
    group_query = new_group_query()
    group_query2 |= group_query
    assert len(group_query2) == len(group_query)
    assert isinstance(group_query, exp.GroupQuery)

    group_query -= group_query2  # pyright: ignore[reportOperatorIssue]
    assert next(iter(group_query2)) not in group_query
    assert isinstance(group_query, exp.GroupQuery)

    group_query &= group_query2  # pyright: ignore[reportOperatorIssue]
    assert len(group_query) == 0
    assert isinstance(group_query, exp.GroupQuery)


@pytest.mark.no_rpm_mock
def test_group(
    default_backend: BackendMod, new_group_query: Callable[..., GroupQuery]
) -> None:
    experimental = default_backend_experimental(default_backend)
    group_query = new_group_query()

    group = next(iter(group_query.filterm(groupid="container-management")))
    assert group.name == "Container Management"
    assert group.translated_name == group.name
    assert group.description == "Tools for managing Linux containers"
    assert group.translated_description == group.description
    assert not group.langonly
    assert group.uservisible
    # Check __hash__
    group_set = set(group_query)
    assert len(group_set) == len(group_query)

    packages = sorted(group.packages)
    defaults = group.get_packages_of_type(experimental.PackageType.DEFAULT)
    assert [package.name for package in defaults] == ["podman"]
    defaults = group.get_packages_of_type(~experimental.PackageType.OPTIONAL)
    assert [package.name for package in defaults] == ["podman"]

    podman = defaults[0]
    buildah = next(package for package in group.packages if package.name == "buildah")
    assert buildah < podman
    assert buildah <= podman
    assert buildah == buildah  # noqa: PLR0124
    assert podman > buildah
    assert podman >= buildah
    assert podman != buildah

    # Check __hash__
    package_set = set(packages)
    assert len(package_set) == len(packages)

    packages2 = sorted(
        group.get_packages_of_type(
            experimental.PackageType.DEFAULT | experimental.PackageType.OPTIONAL
        )
    )
    # The group only contains DEFAULT and OPTIONAL packages, so these should be
    # the same
    assert packages2 == packages


@pytest.mark.no_rpm_mock
def test_environment(
    default_backend: BackendMod, new_env_query: Callable[..., EnvironmentQuery]
) -> None:
    exp = default_backend_experimental(default_backend)
    env_query = new_env_query()
    env_query2 = new_env_query()

    env_query.filter_environmentid("*-desktop-*", exp.QueryCmp.GLOB)
    assert all(e.environmentid.endswith("-desktop-environment") for e in env_query)
    env_query2.filterm(name="Basic Desktop")
    assert len(env_query2) == 1

    env_query.filterm(name=["Basic Desktop", "Budgie Desktop"])
    assert len(env_query) == 2
    q_sorted = sorted(env_query)
    basic = q_sorted[0]
    assert basic.environmentid == "basic-desktop-environment"
    budgie = q_sorted[1]
    assert budgie.environmentid == "budgie-desktop-environment"
    assert basic < budgie
    assert basic <= budgie
    assert basic == basic  # noqa: PLR0124
    assert budgie > basic
    assert budgie >= basic
    assert budgie != basic

    # Test the methods for set operations here
    # The binary operators are tested above
    env_query = env_query.intersection(env_query2)
    env = next(iter(env_query))
    # assert env == basic  # TODO: This doesn't work with libdnf5
    assert len(env_query) == 1
    assert env.name == "Basic Desktop"
    assert env.translated_name == env.name
    assert env.description == "X Window System with a choice of window manager."
    assert env.translated_description == env.description
    assert env in env_query

    old_len = len(env_query2)
    env_query2.remove(env)
    assert env not in env_query2
    assert len(env_query2) == old_len - 1
    env_query2.add(env)

    env_query2 = env_query2.difference(env_query)
    assert env not in env_query2
    assert len(env_query2) == old_len - 1
    env_query2.add(env)

    env_query_union = env_query.union(env_query2)
    env_query.update(env_query2)
    assert set(env_query) == set(env_query2) == set(env_query_union)

    for groups in (env.groupids, env.optional_groupids):
        firstgroup = next(iter(groups))
        assert isinstance(firstgroup, str)
        assert " " not in firstgroup  # It should be a space-less groupid

    assert env_query
    env_query.clear()
    assert not env_query



# Copyright (C) 2024 Maxwell G <maxwell@gtmx.me>
# SPDX-License-Identifier: GPL-2.0-or-later


"""
This module (and the other experimental modules in fedrq) are not meant for public use.
They are subject to breaking changes in minor releases and should not be relied
on by external code.
Once the functionality has stabilized, this code will be moved out of the
experimental namespace.
"""

from __future__ import annotations

from collections.abc import Iterator
from enum import IntEnum
from typing import TYPE_CHECKING, Any

import libdnf5
import libdnf5._comps
import libdnf5.common
import libdnf5.comps

# from . import libdnf5
from fedrq.backends.base import experimental as base

if TYPE_CHECKING:
    from typing_extensions import Self, Unpack


class PackageType(IntEnum):
    CONDITIONAL = libdnf5.comps.PackageType_CONDITIONAL
    DEFAULT = libdnf5.comps.PackageType_DEFAULT
    MANDATORY = libdnf5.comps.PackageType_MANDATORY
    OPTIONAL = libdnf5.comps.PackageType_OPTIONAL

    @classmethod
    def _from_int(cls, value: Any) -> PackageType:
        return {i: i for i in cls}[value]


class GroupPackage(libdnf5.comps.Package, base.GroupPackage):
    @property
    def name(self) -> str:
        return self.get_name()

    @property
    def package_type(self) -> PackageType:
        return PackageType._from_int(self.get_type())

    def __gt__(self, other: Self) -> bool:
        return self._comptup() > other._comptup()

    def __ge__(self, other: Self) -> bool:
        return self._comptup() >= other._comptup()

    def __lt__(self, other: Self) -> bool:
        return self._comptup() < other._comptup()

    def __le__(self, other: Self) -> bool:
        return self._comptup() <= other._comptup()

    def _comptup(self) -> tuple[Any, ...]:
        return (self.name, self.package_type)

    if not getattr(libdnf5.comps.Package, "__hash__", None):

        def __hash__(self) -> int:
            return hash((self.name, self.package_type))


libdnf5._comps.Package_swigregister(GroupPackage)


class Group(libdnf5.comps.Group, base.Group):
    __lt__ = libdnf5.comps.Group.__lt__

    def __le__(self, other: Self) -> bool:
        return not (self > other)

    def __gt__(self, other: Self) -> bool:
        # Based on operator< impl in libdnf5/comps/group/group.cpp
        return self.groupid > other.groupid or self.get_repos() > other.get_repos()

    def __ge__(self, other: Self) -> bool:
        return not (self < other)

    @property
    def groupid(self) -> str:
        return self.get_groupid()

    @property
    def name(self) -> str:
        return self.get_name()

    @property
    def description(self) -> str:
        return self.get_description()

    @property
    def translated_name(self) -> str:
        return self.get_translated_name()

    @property
    def translated_description(self) -> str:
        return self.get_translated_description()

    @property
    def langonly(self) -> bool:
        return self.get_langonly()

    @property
    def uservisible(self) -> bool:
        return self.get_uservisible()

    @property
    def packages(self) -> base.SimpleSequence[GroupPackage]:
        return self.get_packages()

    if not getattr(libdnf5.comps.Group, "__hash__", None):

        def __hash__(self) -> int:
            return hash((self.groupid, self.get_repos()))

    # Add annotations for methods provided by parent
    if TYPE_CHECKING:

        def get_packages(self) -> list[GroupPackage]:
            return []


libdnf5._comps.Group_swigregister(Group)


class QueryCmp(IntEnum):
    EQ = libdnf5.common.QueryCmp_EQ
    NEQ = libdnf5.common.QueryCmp_NEQ
    CONTAINS = libdnf5.common.QueryCmp_CONTAINS
    GLOB = libdnf5.common.QueryCmp_GLOB


class GroupQuery(libdnf5.comps.GroupQuery, base.GroupQuery):

    def difference(self, other) -> GroupQuery:
        """
        NOTE: Unlike normal Python sets, this modifies 'self' in place.
        """
        libdnf5.comps.GroupQuery.difference(self, other)
        return self

    __isub__ = difference

    def union(self, other) -> GroupQuery:
        """
        NOTE: This modifies 'self' in place.
        """
        self.update(other)
        return self

    __ior__ = union

    def intersection(self, other) -> GroupQuery:
        """
        NOTE: Unlike normal Python sets, this modifies 'self' in place.
        """
        libdnf5.comps.GroupQuery.intersection(self, other)
        return self

    __iand__ = intersection

    def __contains__(self, other) -> bool:
        return self.contains(other)

    def __len__(self) -> int:
        return self.size()

    def __bool__(self) -> bool:
        return not self.empty()

    def filterm(self, **kwargs: Unpack[base.GroupQueryKwargs]) -> GroupQuery:
        base._filter_func(self, QueryCmp, **kwargs)
        return self

    if TYPE_CHECKING:

        # We register our subclass with swig so that the iterator contains our
        # Group subclass instead of the base Group class.
        # Let the type checker know that.
        def __iter__(  # pyright: ignore[reportIncompatibleMethodOverride]
            self,
        ) -> Iterator[Group]:
            return
            yield


libdnf5._comps.GroupQuery_swigregister(GroupQuery)


class Environment(libdnf5.comps.Environment, base.Environment):
    __lt__ = libdnf5.comps.Environment.__lt__

    def __le__(self, other: Self) -> bool:
        return not (self > other)

    def __gt__(self, other: Self) -> bool:
        # Based on operator< impl in libdnf5/comps/group/group.cpp
        return (
            self.environmentid > other.environmentid
            or self.get_repos() > other.get_repos()
        )

    def __ge__(self, other: Self) -> bool:
        return not (self < other)

    @property
    def environmentid(self) -> str:
        return self.get_environmentid()

    @property
    def name(self) -> str:
        return self.get_name()

    @property
    def description(self) -> str:
        return self.get_description()

    @property
    def translated_name(self) -> str:
        return self.get_translated_name()

    @property
    def translated_description(self) -> str:
        return self.get_translated_description()

    @property
    def groupids(self) -> base.SimpleSequence[str]:
        return self.get_groups()

    @property
    def optional_groupids(self) -> base.SimpleSequence[str]:
        return self.get_optional_groups()

    def __hash__(self) -> int:
        return hash((self.environmentid, tuple(self.get_repos())))


libdnf5._comps.Environment_swigregister(Environment)


class EnvironmentQuery(libdnf5.comps.EnvironmentQuery, base.EnvironmentQuery):

    def difference(self, other) -> EnvironmentQuery:
        """
        NOTE: Unlike normal Python sets, this modifies 'self' in place.
        """
        libdnf5.comps.EnvironmentQuery.difference(self, other)
        return self

    __isub__ = difference

    def union(self, other) -> EnvironmentQuery:
        """
        NOTE: This modifies 'self' in place.
        """
        self.update(other)
        return self

    __ior__ = union

    def intersection(self, other) -> EnvironmentQuery:
        """
        NOTE: Unlike normal Python sets, this modifies 'self' in place.
        """
        libdnf5.comps.EnvironmentQuery.intersection(self, other)
        return self

    __iand__ = intersection

    def __contains__(self, other) -> bool:
        return self.contains(other)

    def __len__(self) -> int:
        return self.size()

    def __bool__(self) -> bool:
        return not self.empty()

    def filterm(
        self, **kwargs: Unpack[base.EnvironmentQueryKwargs]
    ) -> EnvironmentQuery:
        base._filter_func(self, QueryCmp, **kwargs)
        return self

    if TYPE_CHECKING:

        # We register our subclass with swig so that the iterator contains our
        # Environment subclass instead of the base Environment class.
        # Let the type checker know that.
        def __iter__(  # pyright: ignore[reportIncompatibleMethodOverride]
            self,
        ) -> Iterator[Environment]:
            return
            yield


libdnf5._comps.EnvironmentQuery_swigregister(EnvironmentQuery)



"""
This type stub file was generated by pyright.
"""

from collections.abc import Iterator

if __package__ or "." in __name__:
    ...
else:
    ...

class _SwigNonDynamicMeta(type):
    """Meta class to enforce nondynamic attributes (no new attributes) for a class"""

    __setattr__ = ...

class SwigPyIterator:
    thisown = ...
    def __init__(self, *args, **kwargs) -> None: ...

    __repr__ = ...
    __swig_destroy__ = ...
    def value(self): ...
    def incr(self, n=...): ...
    def decr(self, n=...): ...
    def distance(self, x): ...
    def equal(self, x): ...
    def copy(self): ...
    def next(self): ...
    def __next__(self): ...
    def previous(self): ...
    def advance(self, n): ...
    def __eq__(self, x) -> bool: ...
    def __ne__(self, x) -> bool: ...
    def __iadd__(self, n): ...
    def __isub__(self, n): ...
    def __add__(self, n): ...
    def __sub__(self, *args): ...
    def __iter__(self):  # -> Self:
        ...

PackageType_CONDITIONAL: int = ...
PackageType_DEFAULT: int = ...
PackageType_MANDATORY: int = ...
PackageType_OPTIONAL: int = ...

class InvalidPackageType:
    thisown = ...
    __repr__ = ...
    def __init__(self, *args) -> None: ...
    def get_domain_name(self): ...
    def get_name(self): ...

    __swig_destroy__ = ...

def __or__(a, b): ...
def __ior__(a, b): ...
def __and__(a, b): ...
def any(flags): ...
def package_type_from_string(*args): ...
def package_type_to_string(type): ...
def package_types_to_strings(types): ...

class Package:
    thisown = ...
    __repr__ = ...
    def __init__(self, name, type, condition) -> None: ...
    def __eq__(self, other) -> bool: ...
    def __ne__(self, other) -> bool: ...
    def get_name(self):
        r"""
        :rtype: string
        :return: The Package name.
        Since: 5.0
        """
        ...

    def set_name(self, value): ...
    def get_type(self):
        r"""
        :rtype: int
        :return: The PackageType.
        Since: 5.0
        """
        ...

    def set_type(self, value): ...
    def get_type_string(self):
        r"""
        :rtype: string
        :return: std::string that corresponds to the PackageType.
        Since: 5.0
        """
        ...

    def get_condition(self):
        r"""
        :rtype: string
        :return: The condition (name of package) under which the package gets installed.
        Since: 5.0
        """
        ...

    def set_condition(self, value): ...

    __swig_destroy__ = ...

class GroupId:
    thisown = ...
    __repr__ = ...
    def __init__(self, *args) -> None: ...
    def __eq__(self, other) -> bool: ...
    def __ne__(self, other) -> bool: ...

    id = ...
    __swig_destroy__ = ...

class Group:
    thisown = ...
    def __init__(self, *args, **kwargs) -> None: ...

    __repr__ = ...
    def get_groupid(self):
        r"""
        :rtype: string
        :return: The Group id.
        Since: 5.0
        """
        ...

    def get_name(self):
        r"""
        :rtype: string
        :return: The Group name.
        Since: 5.0
        """
        ...

    def get_description(self):
        r"""
        :rtype: string
        :return: The Group description.
        Since: 5.0
        """
        ...

    def get_translated_name(self, *args): ...
    def get_translated_description(self, *args): ...
    def get_order(self):
        r"""
        :rtype: string
        :return: The Group display order.
        Since: 5.0
        """
        ...

    def get_langonly(self):
        r"""
        :rtype: string
        :return: The Group langonly.
        Since: 5.0
        """
        ...

    def get_uservisible(self):
        r"""
        :rtype: boolean
        :return: `true` if the Group is visible to the users.
        Since: 5.0
        """
        ...

    def get_default(self):
        r"""
        :rtype: boolean
        :return: `true` if the Group is installed by default.
        Since: 5.0
        """
        ...

    def get_packages(self):
        r"""
        :rtype: std::vector< libdnf5::comps::Package,std::allocator< libdnf5::comps::Package > >
        :return: std::vector of Packages belonging to the Group.
        Since: 5.0
        """
        ...

    def get_packages_of_type(self, type):
        r"""
        :rtype: std::vector< libdnf5::comps::Package,std::allocator< libdnf5::comps::Package > >
        :return: std::vector of Packages of given type belonging to the Group.
        :type type: int
        :param type: One of the PackageTypes.
        Since: 5.0
        """
        ...

    def get_repos(self):
        r"""
        :rtype: std::set< std::string,std::less< std::string >,std::allocator< std::string > >
        :return: std::set of names of repositories that contain the Group.
        Since: 5.0
        """
        ...

    def get_installed(self):
        r"""
        :rtype: boolean
        :return: `true` if the Group is installed (belongs to the @System repo).
        Since: 5.0
        """
        ...

    def get_reason(self):
        r"""
        :rtype: int
        :return: Resolved reason why the Group was installed.
                    Groups can be installed due to multiple reasons, only the most significant is returned.
        Since: 5.0
        """
        ...

    def __iadd__(self, rhs):
        r"""
        Merge the Group with another one.
        Since: 5.0
        """
        ...

    def __eq__(self, rhs) -> bool: ...
    def __ne__(self, rhs) -> bool: ...
    def __lt__(self, rhs) -> bool: ...
    def serialize(self, path):
        r"""
        Serialize the Group into an xml file.
        :type path: string
        :param path: Path of the output xml file.
        :raises: utils::xml::XMLSaveError When saving of the file fails.
        Since: 5.0
        """
        ...
    __swig_destroy__ = ...

class VectorPackage:
    thisown = ...
    __repr__ = ...
    def iterator(self): ...
    def __iter__(self): ...
    def __nonzero__(self): ...
    def __bool__(self): ...
    def __len__(self): ...
    def __getslice__(self, i, j): ...
    def __setslice__(self, *args): ...
    def __delslice__(self, i, j): ...
    def __delitem__(self, *args): ...
    def __getitem__(self, *args): ...
    def __setitem__(self, *args): ...
    def pop(self): ...
    def append(self, x): ...
    def empty(self): ...
    def size(self): ...
    def swap(self, v): ...
    def begin(self): ...
    def end(self): ...
    def rbegin(self): ...
    def rend(self): ...
    def clear(self): ...
    def get_allocator(self): ...
    def pop_back(self): ...
    def erase(self, *args): ...
    def __init__(self, *args) -> None: ...
    def push_back(self, x): ...
    def front(self): ...
    def back(self): ...
    def assign(self, n, x): ...
    def insert(self, *args): ...
    def reserve(self, n): ...
    def capacity(self): ...

    __swig_destroy__ = ...

class SetConstIteratorGroup:
    thisown = ...
    def __init__(self, *args, **kwargs) -> None: ...

    __repr__ = ...
    def value(self): ...
    def next(self): ...
    def prev(self): ...
    def __eq__(self, other) -> bool: ...
    def __ne__(self, other) -> bool: ...

    __swig_destroy__ = ...

class SetGroup:
    r"""
    Set represents set of objects (e.g. repositories, or groups)
    and implements set operations such as unions or differences.
    """

    thisown = ...
    def __init__(self, *args, **kwargs) -> None: ...

    __repr__ = ...
    __swig_destroy__ = ...
    def empty(self): ...
    def size(self): ...
    def clear(self): ...
    def add(self, *args): ...
    def remove(self, obj): ...
    def find(self, obj): ...
    def contains(self, obj): ...
    def __ior__(self, other): ...
    def __iand__(self, other): ...
    def __isub__(self, other): ...
    def __ixor__(self, other): ...
    def update(self, other): ...
    def intersection(self, other): ...
    def difference(self, other): ...
    def symmetric_difference(self, other): ...
    def is_subset(self, other): ...
    def is_superset(self, other): ...
    def swap(self, other): ...
    def get_data(self, *args): ...
    def begin(self): ...
    def end(self): ...
    def __len__(self): ...
    def __iter__(self) -> Iterator[Group]: ...

class SackQueryGroup(SetGroup):
    r"""Query is a Set with filtering capabilities."""

    thisown = ...
    __repr__ = ...
    def __init__(self, *args) -> None: ...
    def filter(self, *args): ...
    def get(self):
        r"""Get a single object. Raise an exception if none or multiple objects match the query."""
        ...

    def list(self):
        r"""List all objects matching the query."""
        ...

    def get_data(self, *args): ...

    __swig_destroy__ = ...

class GroupQuery(SackQueryGroup):
    thisown = ...
    __repr__ = ...
    def __init__(self, *args) -> None: ...
    def filter_groupid(self, *args): ...
    def filter_package_name(self, *args):
        r"""
        Filter groups by packages they contain. Keep only groups that contain packages with given names.

        :type patterns: std::vector< std::string,std::allocator< std::string > >
        :param patterns:         A vector of strings (package names) the filter is matched against.
        :param cmp_type:         A comparison (match) operator, defaults to `QueryCmp::EQ`.
        Since: 5.12
        """
        ...

    def filter_name(self, *args): ...
    def filter_uservisible(self, value): ...
    def filter_default(self, value): ...
    def filter_installed(self, value): ...

    __swig_destroy__ = ...

class SackGroup:
    thisown = ...
    def __init__(self, *args, **kwargs) -> None: ...

    __repr__ = ...
    def size(self): ...
    def get_excludes(self): ...
    def add_excludes(self, value): ...
    def remove_excludes(self, value): ...
    def set_excludes(self, value): ...
    def get_includes(self): ...
    def add_includes(self, value): ...
    def remove_includes(self, value): ...
    def set_includes(self, value): ...
    def get_use_includes(self): ...
    def set_use_includes(self, value): ...

    __swig_destroy__ = ...

class GroupSack(SackGroup):
    thisown = ...
    def __init__(self, *args, **kwargs) -> None: ...

    __repr__ = ...
    __swig_destroy__ = ...
    def get_weak_ptr(self):
        r"""Create WeakPtr to GroupSack"""
        ...

    def get_reasons(self): ...

class GroupSackWeakPtr:
    r"""
    WeakPtr is a "smart" pointer. It contains a pointer to resource and to guard of resource.
    WeakPtr pointer can be owner of the resource. However, the resource itself may depend on another resource.
    WeakPtr registers/unregisters itself at the guard of resource. And the resource guard invalidates
    the registered WeakPtrs when the resource is unusable (eg. its dependecny was released).
    Note on thread safety:
    It is safe to create, access and destroy WeakPtrs in multiple threads simultaneously.
    """

    thisown = ...
    __repr__ = ...
    def __init__(self, *args) -> None: ...

    __swig_destroy__ = ...
    def __deref__(self):
        r"""Provides access to the managed object. Generates exception if object is not valid."""
        ...

    def get(self):
        r"""Returns a pointer to the managed object. Generates exception if object is not valid."""
        ...

    def is_valid(self):
        r"""Checks if managed object is valid."""
        ...

    def has_same_guard(self, other):
        r"""Checks if the other WeakPtr instance has the same WeakPtrGuard."""
        ...

    def __ref__(self): ...
    def __eq__(self, other) -> bool: ...
    def __ne__(self, other) -> bool: ...
    def __lt__(self, other) -> bool: ...
    def __gt__(self, other) -> bool: ...
    def __le__(self, other) -> bool: ...
    def __ge__(self, other) -> bool: ...
    def __hash__(self) -> int: ...
    def get_weak_ptr(self):
        r"""Create WeakPtr to GroupSack"""
        ...

    def get_reasons(self): ...
    def size(self): ...
    def get_excludes(self): ...
    def add_excludes(self, value): ...
    def remove_excludes(self, value): ...
    def set_excludes(self, value): ...
    def get_includes(self): ...
    def add_includes(self, value): ...
    def remove_includes(self, value): ...
    def set_includes(self, value): ...
    def get_use_includes(self): ...
    def set_use_includes(self, value): ...

class EnvironmentId:
    thisown = ...
    __repr__ = ...
    def __init__(self, *args) -> None: ...
    def __eq__(self, other) -> bool: ...
    def __ne__(self, other) -> bool: ...

    id = ...
    __swig_destroy__ = ...

class Environment:
    thisown = ...
    def __init__(self, *args, **kwargs) -> None: ...

    __repr__ = ...
    def get_environmentid(self):
        r"""
        :rtype: string
        :return: The Environment id.
        Since: 5.0
        """
        ...

    def get_name(self):
        r"""
        :rtype: string
        :return: The Environment name.
        Since: 5.0
        """
        ...

    def get_description(self):
        r"""
        :rtype: string
        :return: The Environment description.
        Since: 5.0
        """
        ...

    def get_translated_name(self, *args): ...
    def get_translated_description(self, *args): ...
    def get_order(self):
        r"""
        :rtype: string
        :return: The Environment display order.
        Since: 5.0
        """
        ...

    def get_groups(self):
        r"""
        :rtype: std::vector< std::string,std::allocator< std::string > >
        :return: std::vector of Group ids belonging to the Environment.
        Since: 5.0
        """
        ...

    def get_optional_groups(self):
        r"""
        :rtype: std::vector< std::string,std::allocator< std::string > >
        :return: std::vector of optional Group ids belonging to the Environment.
        Since: 5.0
        """
        ...

    def get_repos(self):
        r"""
        :rtype: std::set< std::string,std::less< std::string >,std::allocator< std::string > >
        :return: std::set of names of repositories that contain the Environment.
        Since: 5.0
        """
        ...

    def get_installed(self):
        r"""
        :rtype: boolean
        :return: `true` if the Environment is installed (belongs to the @System repo).
        Since: 5.0
        """
        ...

    def get_reason(self):
        r"""
        :rtype: int
        :return: Resolved reason why the Environment was installed.
                    Environments can be installed due to multiple reasons, only the most significant is returned.
        Since: 5.0
        """
        ...

    def __iadd__(self, rhs):
        r"""
        Merge the Environment with another one.
        Since: 5.0
        """
        ...

    def __eq__(self, rhs) -> bool: ...
    def __ne__(self, rhs) -> bool: ...
    def __lt__(self, rhs) -> bool: ...
    def serialize(self, path):
        r"""
        Serialize the Environment into an xml file
        :type path: string
        :param path: Path of the output xml file.
        :raises: utils::xml::XMLSaveError When saving of the file fails.
        Since: 5.0
        """
        ...
    __swig_destroy__ = ...

class SetConstIteratorEnvironment:
    thisown = ...
    def __init__(self, *args, **kwargs) -> None: ...

    __repr__ = ...
    def value(self): ...
    def next(self): ...
    def prev(self): ...
    def __eq__(self, other) -> bool: ...
    def __ne__(self, other) -> bool: ...

    __swig_destroy__ = ...

class SetEnvironment:
    r"""
    Set represents set of objects (e.g. repositories, or groups)
    and implements set operations such as unions or differences.
    """

    thisown = ...
    def __init__(self, *args, **kwargs) -> None: ...

    __repr__ = ...
    __swig_destroy__ = ...
    def empty(self): ...
    def size(self): ...
    def clear(self): ...
    def add(self, *args): ...
    def remove(self, obj): ...
    def find(self, obj): ...
    def contains(self, obj): ...
    def __ior__(self, other): ...
    def __iand__(self, other): ...
    def __isub__(self, other): ...
    def __ixor__(self, other): ...
    def update(self, other): ...
    def intersection(self, other): ...
    def difference(self, other): ...
    def symmetric_difference(self, other): ...
    def is_subset(self, other): ...
    def is_superset(self, other): ...
    def swap(self, other): ...
    def get_data(self, *args): ...
    def begin(self): ...
    def end(self): ...
    def __len__(self): ...

def __eq__(*args) -> bool: ...

class SackQueryEnvironment(SetEnvironment):
    r"""Query is a Set with filtering capabilities."""

    thisown = ...
    __repr__ = ...
    def __init__(self, *args) -> None: ...
    def filter(self, *args): ...
    def get(self):
        r"""Get a single object. Raise an exception if none or multiple objects match the query."""
        ...

    def list(self):
        r"""List all objects matching the query."""
        ...

    def get_data(self, *args): ...

    __swig_destroy__ = ...

class EnvironmentQuery(SackQueryEnvironment):
    thisown = ...
    __repr__ = ...
    def __init__(self, *args) -> None: ...
    def filter_environmentid(self, *args): ...
    def filter_name(self, *args): ...
    def filter_installed(self, value): ...

    __swig_destroy__ = ...

class SackEnvironment:
    thisown = ...
    def __init__(self, *args, **kwargs) -> None: ...

    __repr__ = ...
    def size(self): ...
    def get_excludes(self): ...
    def add_excludes(self, value): ...
    def remove_excludes(self, value): ...
    def set_excludes(self, value): ...
    def get_includes(self): ...
    def add_includes(self, value): ...
    def remove_includes(self, value): ...
    def set_includes(self, value): ...
    def get_use_includes(self): ...
    def set_use_includes(self, value): ...

    __swig_destroy__ = ...

class EnvironmentSack(SackEnvironment):
    thisown = ...
    def __init__(self, *args, **kwargs) -> None: ...

    __repr__ = ...
    __swig_destroy__ = ...
    def get_weak_ptr(self):
        r"""Create WeakPtr to EnvironmentSack"""
        ...

    def get_reasons(self): ...

class EnvironmentSackWeakPtr:
    r"""
    WeakPtr is a "smart" pointer. It contains a pointer to resource and to guard of resource.
    WeakPtr pointer can be owner of the resource. However, the resource itself may depend on another resource.
    WeakPtr registers/unregisters itself at the guard of resource. And the resource guard invalidates
    the registered WeakPtrs when the resource is unusable (eg. its dependecny was released).
    Note on thread safety:
    It is safe to create, access and destroy WeakPtrs in multiple threads simultaneously.
    """

    thisown = ...
    __repr__ = ...
    def __init__(self, *args) -> None: ...

    __swig_destroy__ = ...
    def __deref__(self):
        r"""Provides access to the managed object. Generates exception if object is not valid."""
        ...

    def get(self):
        r"""Returns a pointer to the managed object. Generates exception if object is not valid."""
        ...

    def is_valid(self):
        r"""Checks if managed object is valid."""
        ...

    def has_same_guard(self, other):
        r"""Checks if the other WeakPtr instance has the same WeakPtrGuard."""
        ...

    def __ref__(self): ...
    def __eq__(self, other) -> bool: ...
    def __ne__(self, other) -> bool: ...
    def __lt__(self, other) -> bool: ...
    def __gt__(self, other) -> bool: ...
    def __le__(self, other) -> bool: ...
    def __ge__(self, other) -> bool: ...
    def __hash__(self) -> int: ...
    def get_weak_ptr(self):
        r"""Create WeakPtr to EnvironmentSack"""
        ...

    def get_reasons(self): ...
    def size(self): ...
    def get_excludes(self): ...
    def add_excludes(self, value): ...
    def remove_excludes(self, value): ...
    def set_excludes(self, value): ...
    def get_includes(self): ...
    def add_includes(self, value): ...
    def remove_includes(self, value): ...
    def set_includes(self, value): ...
    def get_use_includes(self): ...
    def set_use_includes(self, value): ...

def SetEnvironment__iter__(self):  # -> Iterator:
    ...

class Comps:
    thisown = ...
    __repr__ = ...
    def __init__(self, base) -> None: ...

    __swig_destroy__ = ...
    def get_group_sack(self): ...
    def get_environment_sack(self): ...
    def get_weak_ptr(self): ...
    def get_base(self): ...

class CompsWeakPtr:
    r"""
    WeakPtr is a "smart" pointer. It contains a pointer to resource and to guard of resource.
    WeakPtr pointer can be owner of the resource. However, the resource itself may depend on another resource.
    WeakPtr registers/unregisters itself at the guard of resource. And the resource guard invalidates
    the registered WeakPtrs when the resource is unusable (eg. its dependecny was released).
    Note on thread safety:
    It is safe to create, access and destroy WeakPtrs in multiple threads simultaneously.
    """

    thisown = ...
    __repr__ = ...
    def __init__(self, *args) -> None: ...

    __swig_destroy__ = ...
    def __deref__(self):
        r"""Provides access to the managed object. Generates exception if object is not valid."""
        ...

    def get(self):
        r"""Returns a pointer to the managed object. Generates exception if object is not valid."""
        ...

    def is_valid(self):
        r"""Checks if managed object is valid."""
        ...

    def has_same_guard(self, other):
        r"""Checks if the other WeakPtr instance has the same WeakPtrGuard."""
        ...

    def __ref__(self): ...
    def __eq__(self, other) -> bool: ...
    def __ne__(self, other) -> bool: ...
    def __lt__(self, other) -> bool: ...
    def __gt__(self, other) -> bool: ...
    def __le__(self, other) -> bool: ...
    def __ge__(self, other) -> bool: ...
    def __hash__(self) -> int: ...
    def get_group_sack(self): ...
    def get_environment_sack(self): ...
    def get_weak_ptr(self): ...
    def get_base(self): ...



"""
This type stub file was generated by pyright.
"""

from typing import Any

if __package__ or "." in __name__:
    ...
else:
    ...

class _SwigNonDynamicMeta(type):
    """Meta class to enforce nondynamic attributes (no new attributes) for a class"""

    __setattr__ = ...

class SwigPyIterator:
    thisown = ...
    def __init__(self, *args, **kwargs) -> None: ...

    __repr__ = ...
    __swig_destroy__ = ...
    def value(self): ...
    def incr(self, n=...): ...
    def decr(self, n=...): ...
    def distance(self, x): ...
    def equal(self, x): ...
    def copy(self): ...
    def next(self): ...
    def __next__(self): ...
    def previous(self): ...
    def advance(self, n): ...
    def __eq__(self, x) -> bool: ...
    def __ne__(self, x) -> bool: ...
    def __iadd__(self, n): ...
    def __isub__(self, n): ...
    def __add__(self, n): ...
    def __sub__(self, *args): ...
    def __iter__(self):  # -> Self:
        ...

class StringUniquePtr:
    thisown = ...
    __repr__ = ...
    def __init__(self, *args) -> None: ...
    def __deref__(self): ...
    def release(self): ...
    def reset(self, __p=...): ...
    def swap(self, __u): ...
    def get(self): ...
    def __nonzero__(self): ...

    __bool__ = ...
    __swig_destroy__ = ...

class LibraryVersion:
    r"""
    Library version
    Since: 5.0
    """

    thisown = ...
    __repr__ = ...
    major = ...
    minor = ...
    micro = ...
    def __init__(self) -> None: ...

    __swig_destroy__ = ...

class PluginAPIVersion:
    r"""
    Plugin API version
    Since: 5.0
    """

    thisown = ...
    __repr__ = ...
    major = ...
    minor = ...
    def __init__(self) -> None: ...

    __swig_destroy__ = ...

def get_library_version():
    r"""
    :rtype: :py:class:`LibraryVersion`
    :return: Library version
    Since: 5.0
    """
    ...

def get_plugin_api_version():
    r"""
    :rtype: :py:class:`PluginAPIVersion`
    :return: Plugin API version implemented in the library
    Since: 5.0
    """
    ...

class Option:
    r"""Option class is an abstract class. Parent of all options. Options are used to store a configuration."""

    thisown = ...
    def __init__(self, *args, **kwargs) -> None: ...

    __repr__ = ...
    Priority_EMPTY = ...
    Priority_DEFAULT = ...
    Priority_MAINCONFIG = ...
    Priority_AUTOMATICCONFIG = ...
    Priority_REPOCONFIG = ...
    Priority_INSTALLROOT = ...
    Priority_PLUGINDEFAULT = ...
    Priority_PLUGINCONFIG = ...
    Priority_COMMANDLINE = ...
    Priority_RUNTIME = ...
    __swig_destroy__ = ...
    def clone(self):
        r"""Makes copy (clone) of this object."""
        ...

    def get_priority(self):
        r"""Returns priority (source) of the stored value."""
        ...

    def set(self, *args):
        r"""
        *Overload 1:*
        Parses input string and sets new value and priority (source).
        The value and priority are stored only if the new priority is equal to or higher than the stored priority.

        |

        *Overload 2:*
        Parses input string and sets new value and runtime priority.
        """
        ...

    def get_value_string(self):
        r"""Gets a string representation of the stored value."""
        ...

    def empty(self):
        r"""Checks if the option is empty (has no stored value)."""
        ...

    def lock(self, first_comment):
        r"""
        Locks the option.
        The locked option is read-only. Its value cannot be changed.

        :type first_comment: string
        :param first_comment: The comment will be saved when lock() is first called
        Since: 1.0
        """
        ...

    def is_locked(self):
        r"""
        Checks if the option is locked.

        :rtype: boolean
        :return: 'true' if the option is locked
        Since: 1.0
        """
        ...

    def assert_not_locked(self):
        r"""
        Asserts the option is not locked and throws a `libdnf5::UserAssertionError` in case it is.

        Since: 1.0
        """
        ...

cvar = ...
PLUGIN_API_VERSION = ...
PERSISTDIR = ...
SYSTEM_STATE_DIR = ...
SYSTEM_CACHEDIR = ...
CONF_FILENAME = ...
CONF_DIRECTORY = ...
VERSIONLOCK_CONF_FILENAME = ...
PLUGINS_CONF_DIR = ...
REPOSITORY_CONF_DIRS = ...
REPOS_OVERRIDE_DIR = ...
VARS_DIRS = ...
GROUP_PACKAGE_TYPES = ...
INSTALLONLYPKGS = ...
BUGTRACKER = ...
METADATA_TYPE_COMPS = ...
METADATA_TYPE_FILELISTS = ...
METADATA_TYPE_OTHER = ...
METADATA_TYPE_PRESTO = ...
METADATA_TYPE_UPDATEINFO = ...
OPTIONAL_METADATA_TYPES = ...

class OptionBool(Option):
    r"""
    Option that stores boolean value.
    Supports default value.
    Conversion from string to bool is done according to vectors which contains strings of true and false values.
    Conversion is case insensitive for input. Values must be lower case in vectors.
    """

    thisown = ...
    __repr__ = ...
    def __init__(self, *args) -> None: ...

    __swig_destroy__ = ...
    def clone(self):
        r"""Makes copy (clone) of this object."""
        ...

    def set(self, *args):
        r"""
        *Overload 1:*
        Sets new value and priority (source).
        The value and priority are stored only if the new priority is equal to or higher than the stored priority.

        |

        *Overload 2:*
        Sets new value with the runtime priority.

        |

        *Overload 3:*
        Parses input string and sets new value and priority.
        The value and priority are stored only if the new priority is equal to or higher than the stored priority.

        |

        *Overload 4:*
        Parses input string and sets new value and runtime priority.
        """
        ...

    def get_value(self):
        r"""Gets the stored value."""
        ...

    def get_default_value(self):
        r"""Gets the default value. Default value is used until it is replaced by set() method."""
        ...

    def get_value_string(self):
        r"""Gets a string representation of the stored value."""
        ...

    @staticmethod
    def get_default_true_values():
        r"""Returns vector with default strings used for conversion from string to "true" bool value."""
        ...

    @staticmethod
    def get_default_false_values():
        r"""Returns vector with default strings used for conversion from string to "false" bool value."""
        ...

    def get_true_values(self):
        r"""Returns vector with strings used for conversion from string to "true" bool value."""
        ...

    def get_false_values(self):
        r"""Returns vector with strings used for conversion from string to "false" bool value."""
        ...

    def test(self, arg2):
        r"""Does nothing. But it must be present for compatibility with other option types."""
        ...

    def from_string(self, value):
        r"""Parses input string and returns result."""
        ...

    def to_string(self, value):
        r"""Converts input value to the string."""
        ...

class OptionEnumString(Option):
    r"""
    Option that stores value from enumeration. Specialized template for enumeration of strings.
    It supports default value.
    It supports user defined function for conversion from string.
    """

    thisown = ...
    __repr__ = ...
    def __init__(self, *args) -> None: ...
    def clone(self):
        r"""Makes copy (clone) of this object."""
        ...

    def set(self, *args):
        r"""
        *Overload 1:*
        Parses input string and sets new value and priority.
        The value and priority are stored only if the new priority is equal to or higher than the stored priority.

        |

        *Overload 2:*
        Parses input string and sets new value and runtime priority.
        """
        ...

    def get_value(self):
        r"""Gets the stored value."""
        ...

    def get_default_value(self):
        r"""Gets the default value. Default value is used until it is replaced by set() method."""
        ...

    def get_value_string(self):
        r"""Gets a string representation of the stored value."""
        ...

    def test(self, value):
        r"""Tests input value and throws exception if the value is not allowed."""
        ...

    def from_string(self, value):
        r"""Parses input string and returns result."""
        ...
    __swig_destroy__ = ...

class OptionNumberInt32(Option):
    r"""
    Option that stores numerical value. The type of value is template parameter.
    Support default value, minimal and maximal values, user defined function for conversion from string.
    """

    thisown = ...
    __repr__ = ...
    def __init__(self, *args) -> None:
        r"""
        *Overload 1:*
        Constructor that sets default value and limits for min and max values.

        |

        *Overload 2:*
        Constructor that sets default value and limit for min value.

        |

        *Overload 3:*
        Constructor that sets default value.

        |

        *Overload 4:*
        Constructor that sets default value and limits for min and max values and input parsing function.

        |

        *Overload 5:*
        Constructor that sets default value and limit for min value and input parsing function.

        |

        *Overload 6:*
        Constructor that sets default value and input parsing function.
        """
        ...

    def clone(self):
        r"""Makes copy (clone) of this object."""
        ...

    def set(self, *args):
        r"""
        *Overload 1:*
        Sets new value and priority (source).
        The value and priority are stored only if the new priority is equal to or higher than the stored priority.

        |

        *Overload 2:*
        Sets new value and runtime priority.

        |

        *Overload 3:*
        Parses input string and sets new value and priority.
        The value and priority are stored only if the new priority is equal to or higher than the stored priority.

        |

        *Overload 4:*
        Parses input string and sets new value and runtime priority.
        """
        ...

    def get_value(self):
        r"""Gets the stored value."""
        ...

    def get_default_value(self):
        r"""Gets the default value. Default value is used until it is replaced by set() method."""
        ...

    def get_value_string(self):
        r"""Gets a string representation of the stored value."""
        ...

    def test(self, value):
        r"""Tests input value and throws exception if the value is not allowed."""
        ...

    def from_string(self, value):
        r"""Parses input string and returns result."""
        ...

    def to_string(self, value):
        r"""Converts input value to the string."""
        ...
    __swig_destroy__ = ...

class OptionNumberUInt32(Option):
    r"""
    Option that stores numerical value. The type of value is template parameter.
    Support default value, minimal and maximal values, user defined function for conversion from string.
    """

    thisown = ...
    __repr__ = ...
    def __init__(self, *args) -> None:
        r"""
        *Overload 1:*
        Constructor that sets default value and limits for min and max values.

        |

        *Overload 2:*
        Constructor that sets default value and limit for min value.

        |

        *Overload 3:*
        Constructor that sets default value.

        |

        *Overload 4:*
        Constructor that sets default value and limits for min and max values and input parsing function.

        |

        *Overload 5:*
        Constructor that sets default value and limit for min value and input parsing function.

        |

        *Overload 6:*
        Constructor that sets default value and input parsing function.
        """
        ...

    def clone(self):
        r"""Makes copy (clone) of this object."""
        ...

    def set(self, *args):
        r"""
        *Overload 1:*
        Sets new value and priority (source).
        The value and priority are stored only if the new priority is equal to or higher than the stored priority.

        |

        *Overload 2:*
        Sets new value and runtime priority.

        |

        *Overload 3:*
        Parses input string and sets new value and priority.
        The value and priority are stored only if the new priority is equal to or higher than the stored priority.

        |

        *Overload 4:*
        Parses input string and sets new value and runtime priority.
        """
        ...

    def get_value(self):
        r"""Gets the stored value."""
        ...

    def get_default_value(self):
        r"""Gets the default value. Default value is used until it is replaced by set() method."""
        ...

    def get_value_string(self):
        r"""Gets a string representation of the stored value."""
        ...

    def test(self, value):
        r"""Tests input value and throws exception if the value is not allowed."""
        ...

    def from_string(self, value):
        r"""Parses input string and returns result."""
        ...

    def to_string(self, value):
        r"""Converts input value to the string."""
        ...
    __swig_destroy__ = ...

class OptionNumberInt64(Option):
    r"""
    Option that stores numerical value. The type of value is template parameter.
    Support default value, minimal and maximal values, user defined function for conversion from string.
    """

    thisown = ...
    __repr__ = ...
    def __init__(self, *args) -> None:
        r"""
        *Overload 1:*
        Constructor that sets default value and limits for min and max values.

        |

        *Overload 2:*
        Constructor that sets default value and limit for min value.

        |

        *Overload 3:*
        Constructor that sets default value.

        |

        *Overload 4:*
        Constructor that sets default value and limits for min and max values and input parsing function.

        |

        *Overload 5:*
        Constructor that sets default value and limit for min value and input parsing function.

        |

        *Overload 6:*
        Constructor that sets default value and input parsing function.
        """
        ...

    def clone(self):
        r"""Makes copy (clone) of this object."""
        ...

    def set(self, *args):
        r"""
        *Overload 1:*
        Sets new value and priority (source).
        The value and priority are stored only if the new priority is equal to or higher than the stored priority.

        |

        *Overload 2:*
        Sets new value and runtime priority.

        |

        *Overload 3:*
        Parses input string and sets new value and priority.
        The value and priority are stored only if the new priority is equal to or higher than the stored priority.

        |

        *Overload 4:*
        Parses input string and sets new value and runtime priority.
        """
        ...

    def get_value(self):
        r"""Gets the stored value."""
        ...

    def get_default_value(self):
        r"""Gets the default value. Default value is used until it is replaced by set() method."""
        ...

    def get_value_string(self):
        r"""Gets a string representation of the stored value."""
        ...

    def test(self, value):
        r"""Tests input value and throws exception if the value is not allowed."""
        ...

    def from_string(self, value):
        r"""Parses input string and returns result."""
        ...

    def to_string(self, value):
        r"""Converts input value to the string."""
        ...
    __swig_destroy__ = ...

class OptionNumberUInt64(Option):
    r"""
    Option that stores numerical value. The type of value is template parameter.
    Support default value, minimal and maximal values, user defined function for conversion from string.
    """

    thisown = ...
    __repr__ = ...
    def __init__(self, *args) -> None:
        r"""
        *Overload 1:*
        Constructor that sets default value and limits for min and max values.

        |

        *Overload 2:*
        Constructor that sets default value and limit for min value.

        |

        *Overload 3:*
        Constructor that sets default value.

        |

        *Overload 4:*
        Constructor that sets default value and limits for min and max values and input parsing function.

        |

        *Overload 5:*
        Constructor that sets default value and limit for min value and input parsing function.

        |

        *Overload 6:*
        Constructor that sets default value and input parsing function.
        """
        ...

    def clone(self):
        r"""Makes copy (clone) of this object."""
        ...

    def set(self, *args):
        r"""
        *Overload 1:*
        Sets new value and priority (source).
        The value and priority are stored only if the new priority is equal to or higher than the stored priority.

        |

        *Overload 2:*
        Sets new value and runtime priority.

        |

        *Overload 3:*
        Parses input string and sets new value and priority.
        The value and priority are stored only if the new priority is equal to or higher than the stored priority.

        |

        *Overload 4:*
        Parses input string and sets new value and runtime priority.
        """
        ...

    def get_value(self):
        r"""Gets the stored value."""
        ...

    def get_default_value(self):
        r"""Gets the default value. Default value is used until it is replaced by set() method."""
        ...

    def get_value_string(self):
        r"""Gets a string representation of the stored value."""
        ...

    def test(self, value):
        r"""Tests input value and throws exception if the value is not allowed."""
        ...

    def from_string(self, value):
        r"""Parses input string and returns result."""
        ...

    def to_string(self, value):
        r"""Converts input value to the string."""
        ...
    __swig_destroy__ = ...

class OptionNumberFloat(Option):
    r"""
    Option that stores numerical value. The type of value is template parameter.
    Support default value, minimal and maximal values, user defined function for conversion from string.
    """

    thisown = ...
    __repr__ = ...
    def __init__(self, *args) -> None:
        r"""
        *Overload 1:*
        Constructor that sets default value and limits for min and max values.

        |

        *Overload 2:*
        Constructor that sets default value and limit for min value.

        |

        *Overload 3:*
        Constructor that sets default value.

        |

        *Overload 4:*
        Constructor that sets default value and limits for min and max values and input parsing function.

        |

        *Overload 5:*
        Constructor that sets default value and limit for min value and input parsing function.

        |

        *Overload 6:*
        Constructor that sets default value and input parsing function.
        """
        ...

    def clone(self):
        r"""Makes copy (clone) of this object."""
        ...

    def set(self, *args):
        r"""
        *Overload 1:*
        Sets new value and priority (source).
        The value and priority are stored only if the new priority is equal to or higher than the stored priority.

        |

        *Overload 2:*
        Sets new value and runtime priority.

        |

        *Overload 3:*
        Parses input string and sets new value and priority.
        The value and priority are stored only if the new priority is equal to or higher than the stored priority.

        |

        *Overload 4:*
        Parses input string and sets new value and runtime priority.
        """
        ...

    def get_value(self):
        r"""Gets the stored value."""
        ...

    def get_default_value(self):
        r"""Gets the default value. Default value is used until it is replaced by set() method."""
        ...

    def get_value_string(self):
        r"""Gets a string representation of the stored value."""
        ...

    def test(self, value):
        r"""Tests input value and throws exception if the value is not allowed."""
        ...

    def from_string(self, value):
        r"""Parses input string and returns result."""
        ...

    def to_string(self, value):
        r"""Converts input value to the string."""
        ...
    __swig_destroy__ = ...

class OptionSeconds(OptionNumberInt32):
    r"""
    Option that stores an integer value of seconds.
    Support default value, minimal and maximal values.
    """

    thisown = ...
    __repr__ = ...
    def __init__(self, *args) -> None: ...
    def clone(self):
        r"""Makes copy (clone) of this object."""
        ...

    def set(self, *args):
        r"""
        *Overload 1:*
        Sets new value and priority (source).
        The value and priority are stored only if the new priority is equal to or higher than the stored priority.

        |

        *Overload 2:*
        Sets new value and runtime priority.

        |

        *Overload 3:*
        Parses input string and sets new value and priority.
        Valid inputs: 100, 1.5m, 90s, 1.2d, 1d, 0xF, 0.1, -1, never.
        Invalid inputs: -10, -0.1, 45.6Z, 1d6h, 1day, 1y.
        The value and priority are stored only if the new priority is equal to or higher than the stored priority.

        |

        *Overload 4:*
        Parses input string and sets new value and runtime priority.
        """
        ...

    def from_string(self, value):
        r"""Parses input string and returns result."""
        ...
    __swig_destroy__ = ...

class OptionString(Option):
    r"""
    Option that stores string value.
    Support default value, and check of an input value using the regular expression
    """

    thisown = ...
    __repr__ = ...
    def __init__(self, *args) -> None: ...
    def clone(self):
        r"""Makes copy (clone) of this object."""
        ...

    def set(self, *args):
        r"""
        *Overload 1:*
        Sets new value and priority.
        The value and priority are stored only if the new priority is equal to or higher than the stored priority.

        |

        *Overload 2:*
        Sets new value and runtime priority.
        """
        ...

    def get_value(self):
        r"""Gets the stored value."""
        ...

    def get_default_value(self):
        r"""Gets the default value. Default value is used until it is replaced by set() method."""
        ...

    def get_value_string(self):
        r"""Gets a string representation of the stored value."""
        ...

    def test(self, value):
        r"""Tests input value and throws exception if the value is not allowed."""
        ...

    def from_string(self, value):
        r"""Returns copy of input string. Must be present for compatibility with other option types."""
        ...
    __swig_destroy__ = ...

class OptionStringSet(Option):
    r"""
    Option that stores a container of strings. The type of the container is a template parameter.
    Support default value, and check of an input value using the regular expression
    """

    thisown = ...
    __repr__ = ...
    def __init__(self, *args) -> None: ...

    __swig_destroy__ = ...
    def clone(self):
        r"""Makes copy (clone) of this object."""
        ...

    def set(self, *args):
        r"""
        *Overload 1:*
        Sets new value and priority (source).
        The value and priority are stored only if the new priority is equal to or higher than the stored priority.

        |

        *Overload 2:*
        Sets new value and runtime priority.

        |

        *Overload 3:*
        Parses input string and sets new value and priority.
        The value and priority are stored only if the new priority is equal to or higher than the stored priority.

        |

        *Overload 4:*
        Parses input string and sets new value and runtime priority.
        """
        ...

    def add(self, *args):
        r"""
        *Overload 1:*
        Adds items from an another container.
        New items are stored in the container value

        |

        *Overload 2:*
        Parses input string and adds new values and priority.
        The value and priority are stored only if the new priority is equal to or higher than the stored priority.
        """
        ...

    def add_item(self, priority, item):
        r"""
        Adds new item to the container.
        New item is stored in the container value
        """
        ...

    def get_value(self):
        r"""Gets the stored value."""
        ...

    def get_default_value(self):
        r"""Gets the default value. Default value is used until it is replaced by set() method."""
        ...

    def get_value_string(self):
        r"""Gets a string representation of the stored value."""
        ...

    def test(self, value):
        r"""Tests input value and throws exception if the value is not allowed."""
        ...

    def from_string(self, value):
        r"""Parses input string and returns result."""
        ...

    def to_string(self, value):
        r"""Converts input value to the string."""
        ...

    @staticmethod
    def get_default_delimiters():
        r"""Returns the default delimiters"""
        ...

    def get_delimiters(self):
        r"""Return delimiters of this OptionStringList"""
        ...

class OptionStringList(Option):
    r"""
    Option that stores a container of strings. The type of the container is a template parameter.
    Support default value, and check of an input value using the regular expression
    """

    thisown = ...
    __repr__ = ...
    def __init__(self, *args) -> None: ...

    __swig_destroy__ = ...
    def clone(self):
        r"""Makes copy (clone) of this object."""
        ...

    def set(self, *args):
        r"""
        *Overload 1:*
        Sets new value and priority (source).
        The value and priority are stored only if the new priority is equal to or higher than the stored priority.

        |

        *Overload 2:*
        Sets new value and runtime priority.

        |

        *Overload 3:*
        Parses input string and sets new value and priority.
        The value and priority are stored only if the new priority is equal to or higher than the stored priority.

        |

        *Overload 4:*
        Parses input string and sets new value and runtime priority.
        """
        ...

    def add(self, *args):
        r"""
        *Overload 1:*
        Adds items from an another container.
        New items are stored in the container value

        |

        *Overload 2:*
        Parses input string and adds new values and priority.
        The value and priority are stored only if the new priority is equal to or higher than the stored priority.
        """
        ...

    def add_item(self, priority, item):
        r"""
        Adds new item to the container.
        New item is stored in the container value
        """
        ...

    def get_value(self):
        r"""Gets the stored value."""
        ...

    def get_default_value(self):
        r"""Gets the default value. Default value is used until it is replaced by set() method."""
        ...

    def get_value_string(self):
        r"""Gets a string representation of the stored value."""
        ...

    def test(self, value):
        r"""Tests input value and throws exception if the value is not allowed."""
        ...

    def from_string(self, value):
        r"""Parses input string and returns result."""
        ...

    def to_string(self, value):
        r"""Converts input value to the string."""
        ...

    @staticmethod
    def get_default_delimiters():
        r"""Returns the default delimiters"""
        ...

    def get_delimiters(self):
        r"""Return delimiters of this OptionStringList"""
        ...

class OptionPath(OptionString):
    r"""
    Option that stores file/directory path.
    Support default value, and path verification (absolute, existence).
    """

    thisown = ...
    __repr__ = ...
    def __init__(self, *args) -> None:
        r"""
        *Overload 1:*
        Constructor sets default value and conditions.

        |

        *Overload 2:*
        Constructor sets default value and conditions.

        |

        *Overload 3:*
        Constructor sets default value and conditions.

        |

        *Overload 4:*
        Constructor sets default value and conditions.

        |

        *Overload 5:*
        Constructor sets default value and conditions.

        |

        *Overload 6:*
        Constructor sets default value and conditions.

        |

        *Overload 7:*
        Constructor sets default value and conditions.

        |

        *Overload 8:*
        Constructor sets default value and conditions.

        |

        *Overload 9:*
        Constructor sets default value and conditions.

        |

        *Overload 10:*
        Constructor sets default value and conditions.
        """
        ...

    def clone(self):
        r"""Makes copy (clone) of this object."""
        ...

    def set(self, *args):
        r"""
        *Overload 1:*
        Parses input string and sets new value and priority.
        According setting passed in constructor it can verify that the path is absolute, exists and match regex.
        The value and priority are stored only if the new priority is equal to or higher than the stored priority.

        |

        *Overload 2:*
        Parses input string and sets new value and runtime priority.
        """
        ...

    def test(self, value):
        r"""Tests input value and throws exception if the value is not allowed."""
        ...
    __swig_destroy__ = ...

class OptionChildBool(Option):
    r"""
    Option that links option to another option. It uses default value and parameters from linked option.
    If it is empty (has no stored value), uses value from the linked option (parent).
    Parent option type is template parameter.
    """

    thisown = ...
    __repr__ = ...
    def __init__(self, parent) -> None:
        r"""Constructor takes reference to parent option."""
        ...

    def clone(self):
        r"""Makes copy (clone) of this object."""
        ...

    def get_priority(self):
        r"""Returns priority (source) of the stored value. If no value is stored, priority from the parent is returned."""
        ...

    def set(self, *args):
        r"""
        *Overload 1:*
        Sets new value and priority (source).
        The value and priority are stored only if the new priority is equal to or higher than the stored priority.

        |

        *Overload 2:*
        Sets new value and runtime priority.

        |

        *Overload 3:*
        Sets new value and priority (source).
        The value and priority are stored only if the new priority is equal to or higher than the stored priority.

        |

        *Overload 4:*
        Sets new value and runtime priority.
        """
        ...

    def get_value(self):
        r"""Gets the stored value. If no value is stored, value from the parent is returned."""
        ...

    def get_default_value(self):
        r"""Gets the default value from parent. Default value is used until it is replaced by set() method."""
        ...

    def get_value_string(self):
        r"""Gets a string representation of the stored value. If no value is stored, value from the parent is returned."""
        ...

    def empty(self):
        r"""Checks if the option is empty (has no stored value). If it is empty, checks status of the parent."""
        ...
    __swig_destroy__ = ...

class OptionChildString(Option):
    r"""
    Option that links option to another option. It uses default value and parameters from linked option.
    If it is empty (has no stored value), uses value from the linked option (parent).
    Parent option type is template parameter.
    """

    thisown = ...
    __repr__ = ...
    def __init__(self, parent) -> None:
        r"""Constructor takes reference to parent option."""
        ...

    def clone(self):
        r"""Makes copy (clone) of this object."""
        ...

    def get_priority(self):
        r"""Returns priority (source) of the stored value. If no value is stored, priority from the parent is returned."""
        ...

    def set(self, *args):
        r"""
        *Overload 1:*
        Sets new value and priority (source).
        The value and priority are stored only if the new priority is equal to or higher than the stored priority.

        |

        *Overload 2:*
        Sets new value and runtime priority.

        |

        *Overload 3:*
        Sets new value and priority (source).
        The value and priority are stored only if the new priority is equal to or higher than the stored priority.

        |

        *Overload 4:*
        Sets new value and runtime priority.
        """
        ...

    def get_value(self):
        r"""Gets the stored value. If no value is stored, value from the parent is returned."""
        ...

    def get_default_value(self):
        r"""Gets the default value from parent. Default value is used until it is replaced by set() method."""
        ...

    def get_value_string(self):
        r"""Gets a string representation of the stored value. If no value is stored, value from the parent is returned."""
        ...

    def empty(self):
        r"""Checks if the option is empty (has no stored value). If it is empty, checks status of the parent."""
        ...
    __swig_destroy__ = ...

class OptionChildStringList(Option):
    r"""
    Option that links option to another option. It uses default value and parameters from linked option.
    If it is empty (has no stored value), uses value from the linked option (parent).
    Parent option type is template parameter.
    """

    thisown = ...
    __repr__ = ...
    def __init__(self, parent) -> None:
        r"""Constructor takes reference to parent option."""
        ...

    def clone(self):
        r"""Makes copy (clone) of this object."""
        ...

    def get_priority(self):
        r"""Returns priority (source) of the stored value. If no value is stored, priority from the parent is returned."""
        ...

    def set(self, *args):
        r"""
        *Overload 1:*
        Sets new value and priority (source).
        The value and priority are stored only if the new priority is equal to or higher than the stored priority.

        |

        *Overload 2:*
        Sets new value and runtime priority.

        |

        *Overload 3:*
        Sets new value and priority (source).
        The value and priority are stored only if the new priority is equal to or higher than the stored priority.

        |

        *Overload 4:*
        Sets new value and runtime priority.
        """
        ...

    def get_value(self):
        r"""Gets the stored value. If no value is stored, value from the parent is returned."""
        ...

    def get_default_value(self):
        r"""Gets the default value from parent. Default value is used until it is replaced by set() method."""
        ...

    def get_value_string(self):
        r"""Gets a string representation of the stored value. If no value is stored, value from the parent is returned."""
        ...

    def empty(self):
        r"""Checks if the option is empty (has no stored value). If it is empty, checks status of the parent."""
        ...
    __swig_destroy__ = ...

class OptionChildStringSet(Option):
    r"""
    Option that links option to another option. It uses default value and parameters from linked option.
    If it is empty (has no stored value), uses value from the linked option (parent).
    Parent option type is template parameter.
    """

    thisown = ...
    __repr__ = ...
    def __init__(self, parent) -> None:
        r"""Constructor takes reference to parent option."""
        ...

    def clone(self):
        r"""Makes copy (clone) of this object."""
        ...

    def get_priority(self):
        r"""Returns priority (source) of the stored value. If no value is stored, priority from the parent is returned."""
        ...

    def set(self, *args):
        r"""
        *Overload 1:*
        Sets new value and priority (source).
        The value and priority are stored only if the new priority is equal to or higher than the stored priority.

        |

        *Overload 2:*
        Sets new value and runtime priority.

        |

        *Overload 3:*
        Sets new value and priority (source).
        The value and priority are stored only if the new priority is equal to or higher than the stored priority.

        |

        *Overload 4:*
        Sets new value and runtime priority.
        """
        ...

    def get_value(self):
        r"""Gets the stored value. If no value is stored, value from the parent is returned."""
        ...

    def get_default_value(self):
        r"""Gets the default value from parent. Default value is used until it is replaced by set() method."""
        ...

    def get_value_string(self):
        r"""Gets a string representation of the stored value. If no value is stored, value from the parent is returned."""
        ...

    def empty(self):
        r"""Checks if the option is empty (has no stored value). If it is empty, checks status of the parent."""
        ...
    __swig_destroy__ = ...

class OptionChildNumberInt32(Option):
    r"""
    Option that links option to another option. It uses default value and parameters from linked option.
    If it is empty (has no stored value), uses value from the linked option (parent).
    Parent option type is template parameter.
    """

    thisown = ...
    __repr__ = ...
    def __init__(self, parent) -> None:
        r"""Constructor takes reference to parent option."""
        ...

    def clone(self):
        r"""Makes copy (clone) of this object."""
        ...

    def get_priority(self):
        r"""Returns priority (source) of the stored value. If no value is stored, priority from the parent is returned."""
        ...

    def set(self, *args):
        r"""
        *Overload 1:*
        Sets new value and priority (source).
        The value and priority are stored only if the new priority is equal to or higher than the stored priority.

        |

        *Overload 2:*
        Sets new value and runtime priority.

        |

        *Overload 3:*
        Sets new value and priority (source).
        The value and priority are stored only if the new priority is equal to or higher than the stored priority.

        |

        *Overload 4:*
        Sets new value and runtime priority.
        """
        ...

    def get_value(self):
        r"""Gets the stored value. If no value is stored, value from the parent is returned."""
        ...

    def get_default_value(self):
        r"""Gets the default value from parent. Default value is used until it is replaced by set() method."""
        ...

    def get_value_string(self):
        r"""Gets a string representation of the stored value. If no value is stored, value from the parent is returned."""
        ...

    def empty(self):
        r"""Checks if the option is empty (has no stored value). If it is empty, checks status of the parent."""
        ...
    __swig_destroy__ = ...

class OptionChildNumberUInt32(Option):
    r"""
    Option that links option to another option. It uses default value and parameters from linked option.
    If it is empty (has no stored value), uses value from the linked option (parent).
    Parent option type is template parameter.
    """

    thisown = ...
    __repr__ = ...
    def __init__(self, parent) -> None:
        r"""Constructor takes reference to parent option."""
        ...

    def clone(self):
        r"""Makes copy (clone) of this object."""
        ...

    def get_priority(self):
        r"""Returns priority (source) of the stored value. If no value is stored, priority from the parent is returned."""
        ...

    def set(self, *args):
        r"""
        *Overload 1:*
        Sets new value and priority (source).
        The value and priority are stored only if the new priority is equal to or higher than the stored priority.

        |

        *Overload 2:*
        Sets new value and runtime priority.

        |

        *Overload 3:*
        Sets new value and priority (source).
        The value and priority are stored only if the new priority is equal to or higher than the stored priority.

        |

        *Overload 4:*
        Sets new value and runtime priority.
        """
        ...

    def get_value(self):
        r"""Gets the stored value. If no value is stored, value from the parent is returned."""
        ...

    def get_default_value(self):
        r"""Gets the default value from parent. Default value is used until it is replaced by set() method."""
        ...

    def get_value_string(self):
        r"""Gets a string representation of the stored value. If no value is stored, value from the parent is returned."""
        ...

    def empty(self):
        r"""Checks if the option is empty (has no stored value). If it is empty, checks status of the parent."""
        ...
    __swig_destroy__ = ...

class OptionChildNumberFloat(Option):
    r"""
    Option that links option to another option. It uses default value and parameters from linked option.
    If it is empty (has no stored value), uses value from the linked option (parent).
    Parent option type is template parameter.
    """

    thisown = ...
    __repr__ = ...
    def __init__(self, parent) -> None:
        r"""Constructor takes reference to parent option."""
        ...

    def clone(self):
        r"""Makes copy (clone) of this object."""
        ...

    def get_priority(self):
        r"""Returns priority (source) of the stored value. If no value is stored, priority from the parent is returned."""
        ...

    def set(self, *args):
        r"""
        *Overload 1:*
        Sets new value and priority (source).
        The value and priority are stored only if the new priority is equal to or higher than the stored priority.

        |

        *Overload 2:*
        Sets new value and runtime priority.

        |

        *Overload 3:*
        Sets new value and priority (source).
        The value and priority are stored only if the new priority is equal to or higher than the stored priority.

        |

        *Overload 4:*
        Sets new value and runtime priority.
        """
        ...

    def get_value(self):
        r"""Gets the stored value. If no value is stored, value from the parent is returned."""
        ...

    def get_default_value(self):
        r"""Gets the default value from parent. Default value is used until it is replaced by set() method."""
        ...

    def get_value_string(self):
        r"""Gets a string representation of the stored value. If no value is stored, value from the parent is returned."""
        ...

    def empty(self):
        r"""Checks if the option is empty (has no stored value). If it is empty, checks status of the parent."""
        ...
    __swig_destroy__ = ...

class OptionChildEnumString(Option):
    r"""
    Option that links option to another option. It uses default value and parameters from linked option.
    If it is empty (has no stored value), uses value from the linked option (parent).
    Parent option type is template parameter.
    """

    thisown = ...
    __repr__ = ...
    def __init__(self, parent) -> None:
        r"""Constructor takes reference to parent option."""
        ...

    def clone(self):
        r"""Makes copy (clone) of this object."""
        ...

    def get_priority(self):
        r"""Returns priority (source) of the stored value. If no value is stored, priority from the parent is returned."""
        ...

    def set(self, *args):
        r"""
        *Overload 1:*
        Sets new value and priority (source).
        The value and priority are stored only if the new priority is equal to or higher than the stored priority.

        |

        *Overload 2:*
        Sets new value and runtime priority.

        |

        *Overload 3:*
        Sets new value and priority (source).
        The value and priority are stored only if the new priority is equal to or higher than the stored priority.

        |

        *Overload 4:*
        Sets new value and runtime priority.
        """
        ...

    def get_value(self):
        r"""Gets the stored value. If no value is stored, value from the parent is returned."""
        ...

    def get_default_value(self):
        r"""Gets the default value from parent. Default value is used until it is replaced by set() method."""
        ...

    def get_value_string(self):
        r"""Gets a string representation of the stored value. If no value is stored, value from the parent is returned."""
        ...

    def empty(self):
        r"""Checks if the option is empty (has no stored value). If it is empty, checks status of the parent."""
        ...
    __swig_destroy__ = ...

class OptionChildSeconds(Option):
    r"""
    Option that links option to another option. It uses default value and parameters from linked option.
    If it is empty (has no stored value), uses value from the linked option (parent).
    Parent option type is template parameter.
    """

    thisown = ...
    __repr__ = ...
    def __init__(self, parent) -> None:
        r"""Constructor takes reference to parent option."""
        ...

    def clone(self):
        r"""Makes copy (clone) of this object."""
        ...

    def get_priority(self):
        r"""Returns priority (source) of the stored value. If no value is stored, priority from the parent is returned."""
        ...

    def set(self, *args):
        r"""
        *Overload 1:*
        Sets new value and priority (source).
        The value and priority are stored only if the new priority is equal to or higher than the stored priority.

        |

        *Overload 2:*
        Sets new value and runtime priority.

        |

        *Overload 3:*
        Sets new value and priority (source).
        The value and priority are stored only if the new priority is equal to or higher than the stored priority.

        |

        *Overload 4:*
        Sets new value and runtime priority.
        """
        ...

    def get_value(self):
        r"""Gets the stored value. If no value is stored, value from the parent is returned."""
        ...

    def get_default_value(self):
        r"""Gets the default value from parent. Default value is used until it is replaced by set() method."""
        ...

    def get_value_string(self):
        r"""Gets a string representation of the stored value. If no value is stored, value from the parent is returned."""
        ...

    def empty(self):
        r"""Checks if the option is empty (has no stored value). If it is empty, checks status of the parent."""
        ...
    __swig_destroy__ = ...

class OptionBinds:
    r"""
    Maps the options names (text names read from config file, command line, ...) to options objects.
    Supports user defined functions for processing new value and converting value to string.
    """

    thisown = ...
    __repr__ = ...
    def add(self, id, option): ...
    def at(self, *args): ...
    def empty(self): ...
    def size(self): ...
    def __init__(self) -> None: ...

    __swig_destroy__ = ...

class ConfigParser:
    r"""
    Class for parsing dnf/yum .ini configuration files.

    ConfigParser is used for parsing files.
    User can get both substituted and original parsed values.
    The parsed items are stored into the PreserveOrderMap.
    ConfigParser preserve order of items. Comments and empty lines are kept.
    """

    thisown = ...
    __repr__ = ...
    def read(self, file_path):
        r"""
        Reads/parse one INI file

        Can be called repeately for reading/merge more INI files.

        :type file_path: string
        :param file_path: Name (with path) of file to read
        """
        ...

    def write(self, *args):
        r"""
        *Overload 1:*

        Writes all data (all sections) to INI file

        :type file_path: string
        :param file_path: Name (with path) of file to write
        :type append: boolean
        :param append: If true, existent file will be appended, otherwise overwritten

        |

        *Overload 2:*

        Writes one section data to INI file

        :type file_path: string
        :param file_path: Name (with path) of file to write
        :type append: boolean
        :param append: If true, existent file will be appended, otherwise overwritten
        :type section: string
        :param section: Section to write
        """
        ...

    def add_section(self, *args): ...
    def has_section(self, section): ...
    def has_option(self, section, key): ...
    def set_value(self, *args): ...
    def remove_section(self, section): ...
    def remove_option(self, section, key): ...
    def add_comment_line(self, *args): ...
    def get_value(self, section, key): ...
    def get_header(self, *args): ...
    def get_data(self, *args): ...
    def __init__(self) -> None: ...

    __swig_destroy__ = ...

class Vars:
    r"""
    Class for reading and substituting DNF vars (arch, releasever, etc.).

    The class loads the variables from the environment as well as from a list
    of directories.
    """

    thisown = ...
    __repr__ = ...
    Priority_DEFAULT = ...
    Priority_AUTO = ...
    Priority_VARSDIR = ...
    Priority_PLUGIN = ...
    Priority_ENVIRONMENT = ...
    Priority_COMMANDLINE = ...
    Priority_RUNTIME = ...
    def __init__(self, *args) -> None: ...
    def substitute(self, text):
        r"""
        Substitute DNF vars in the input text.

        :type text: string
        :param text: The text for substitution
        :rtype: string
        :return: The substituted text
        """
        ...

    def get_variables(self): ...
    def set(self, *args):
        r"""
        Set particular variable to a value

        :type name: string
        :param name: Name of the variable
        :type value: string
        :param value: Value to be stored in variable
        :type prio: int, optional
        :param prio: Source/Priority of the value
        :raises: ReadOnlyVariableError if the variable is read-only
        """
        ...

    def is_read_only(self, name):
        r"""
        Checks whether a variable is read-only

        :type name: string
        :param name: Name of the variable
        :rtype: boolean
        :return: true if the variable is read-only, false if it is writable
        """
        ...

    def contains(self, name):
        r"""
        Checks if there is an variable with name equivalent to name in the container.

        :type name: string
        :param name: Name of the variable
        :rtype: boolean
        :return: true if there is such an element, otherwise false
        """
        ...

    def get_value(self, name):
        r"""
        Get value of particular variable.

        :type name: string
        :param name: Name of the variable
        """
        ...

    def get(self, name):
        r"""
        Get particular variable.

        :type name: string
        :param name: Name of the variable
        """
        ...

    @staticmethod
    def detect_release(base, install_root_path): ...

    __swig_destroy__ = ...

class Config:
    r"""Base class for configurations objects"""

    thisown = ...
    __repr__ = ...
    def opt_binds(self): ...

    __swig_destroy__ = ...
    def load_from_parser(self, parser, section, vars, logger, priority): ...
    def __init__(self) -> None: ...

class ConfigMain(Config):
    r"""Holds global configuration"""

    thisown = ...
    __repr__ = ...
    def __init__(self) -> None: ...

    __swig_destroy__ = ...
    def get_debuglevel_option(self, *args): ...
    def get_errorlevel_option(self, *args): ...
    def get_installroot_option(self, *args): ...
    def get_config_file_path_option(self, *args): ...
    def get_plugins_option(self, *args): ...
    def get_pluginpath_option(self, *args): ...
    def get_pluginconfpath_option(self, *args): ...
    def get_persistdir_option(self, *args): ...
    def get_system_state_dir_option(self, *args): ...
    def get_transaction_history_dir_option(self, *args): ...
    def get_transformdb_option(self, *args): ...
    def get_recent_option(self, *args): ...
    def get_reset_nice_option(self, *args): ...
    def get_system_cachedir_option(self, *args): ...
    def get_cacheonly_option(self, *args): ...
    def get_keepcache_option(self, *args): ...
    def get_logdir_option(self, *args): ...
    def get_log_size_option(self, *args): ...
    def get_log_rotate_option(self, *args): ...
    def get_debugdir_option(self, *args): ...
    def get_varsdir_option(self, *args): ...
    def get_reposdir_option(self, *args): ...
    def get_debug_solver_option(self, *args): ...
    def get_installonlypkgs_option(self, *args): ...
    def get_group_package_types_option(self, *args): ...
    def get_optional_metadata_types_option(self, *args): ...
    def get_use_host_config_option(self, *args): ...
    def get_installonly_limit_option(self, *args): ...
    def get_tsflags_option(self, *args): ...
    def get_assumeyes_option(self, *args): ...
    def get_assumeno_option(self, *args): ...
    def get_check_config_file_age_option(self, *args): ...
    def get_defaultyes_option(self, *args): ...
    def get_diskspacecheck_option(self, *args): ...
    def get_localpkg_gpgcheck_option(self, *args): ...
    def get_gpgkey_dns_verification_option(self, *args): ...
    def get_obsoletes_option(self, *args): ...
    def get_exit_on_lock_option(self, *args): ...
    def get_allow_vendor_change_option(self, *args): ...
    def get_metadata_timer_sync_option(self, *args): ...
    def get_disable_excludes_option(self, *args): ...
    def get_multilib_policy_option(self, *args): ...
    def get_best_option(self, *args): ...
    def get_install_weak_deps_option(self, *args): ...
    def get_allow_downgrade_option(self, *args): ...
    def get_bugtracker_url_option(self, *args): ...
    def get_zchunk_option(self, *args): ...
    def get_color_option(self, *args): ...
    def get_color_list_installed_older_option(self, *args): ...
    def get_color_list_installed_newer_option(self, *args): ...
    def get_color_list_installed_reinstall_option(self, *args): ...
    def get_color_list_installed_extra_option(self, *args): ...
    def get_color_list_available_upgrade_option(self, *args): ...
    def get_color_list_available_downgrade_option(self, *args): ...
    def get_color_list_available_reinstall_option(self, *args): ...
    def get_color_list_available_install_option(self, *args): ...
    def get_color_update_installed_option(self, *args): ...
    def get_color_update_local_option(self, *args): ...
    def get_color_update_remote_option(self, *args): ...
    def get_color_search_match_option(self, *args): ...
    def get_history_record_option(self, *args): ...
    def get_history_record_packages_option(self, *args): ...
    def get_rpmverbosity_option(self, *args): ...
    def get_strict_option(self, *args): ...
    def get_skip_broken_option(self, *args): ...
    def get_skip_unavailable_option(self, *args): ...
    def get_autocheck_running_kernel_option(self, *args): ...
    def get_clean_requirements_on_remove_option(self, *args): ...
    def get_history_list_view_option(self, *args): ...
    def get_upgrade_group_objects_upgrade_option(self, *args): ...
    def get_destdir_option(self, *args): ...
    def get_comment_option(self, *args): ...
    def get_downloadonly_option(self, *args): ...
    def get_ignorearch_option(self, *args): ...
    def get_module_platform_id_option(self, *args): ...
    def get_module_stream_switch_option(self, *args): ...
    def get_module_obsoletes_option(self, *args): ...
    def get_user_agent_option(self, *args): ...
    def get_countme_option(self, *args): ...
    def get_protect_running_kernel_option(self, *args): ...
    def get_build_cache_option(self, *args): ...
    def get_retries_option(self, *args): ...
    def get_cachedir_option(self, *args): ...
    def get_fastestmirror_option(self, *args): ...
    def get_excludepkgs_option(self, *args): ...
    def get_includepkgs_option(self, *args): ...
    def get_exclude_from_weak_option(self, *args): ...
    def get_exclude_from_weak_autodetect_option(self, *args): ...
    def get_proxy_option(self, *args): ...
    def get_proxy_username_option(self, *args): ...
    def get_proxy_password_option(self, *args): ...
    def get_proxy_auth_method_option(self, *args): ...
    def get_protected_packages_option(self, *args): ...
    def get_username_option(self, *args): ...
    def get_password_option(self, *args): ...
    def get_gpgcheck_option(self, *args): ...
    def get_repo_gpgcheck_option(self, *args): ...
    def get_enabled_option(self, *args): ...
    def get_enablegroups_option(self, *args): ...
    def get_bandwidth_option(self, *args): ...
    def get_minrate_option(self, *args): ...
    def get_ip_resolve_option(self, *args): ...
    def get_throttle_option(self, *args): ...
    def get_timeout_option(self, *args): ...
    def get_max_parallel_downloads_option(self, *args): ...
    def get_metadata_expire_option(self, *args): ...
    def get_sslcacert_option(self, *args): ...
    def get_sslverify_option(self, *args): ...
    def get_sslclientcert_option(self, *args): ...
    def get_sslclientkey_option(self, *args): ...
    def get_proxy_sslcacert_option(self, *args): ...
    def get_proxy_sslverify_option(self, *args): ...
    def get_proxy_sslclientcert_option(self, *args): ...
    def get_proxy_sslclientkey_option(self, *args): ...
    def get_deltarpm_option(self, *args): ...
    def get_deltarpm_percentage_option(self, *args): ...
    def get_skip_if_unavailable_option(self, *args): ...
    def load_from_parser(self, *args): ...
    # XXX: basedypright does not generate typings for the automatically created
    # properties
    # Use __getattr__/__setattr__ hack for now
    def __getattr__(self, name: str) -> Any: ...
    def __setattr__(self, name: str, value: Any) -> None: ...
